@@ -29,18 +29,18 @@
 
 DO_COMMAND(do_action)
 {
-	char *left, *right, *rank;
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], rank[BUFFER_SIZE];
 	struct listroot *root;
 
 	root = ses->list[LIST_ACTION];
 
-	arg = get_arg_in_braces(arg, &left,  0);
-	arg = get_arg_in_braces(arg, &right, 1);
-	arg = get_arg_in_braces(arg, &rank,  1);
+	arg = get_arg_in_braces(arg, left,  0);
+	arg = get_arg_in_braces(arg, right, 1);
+	arg = get_arg_in_braces(arg, rank,  1);
 
 	if (*rank == 0)
 	{
-		rank = string_alloc("5");
+		strcpy(rank, "5");
 	}
 
 	if (*left == 0)
@@ -74,7 +74,7 @@ DO_COMMAND(do_unaction)
 
 void check_all_actions(struct session *ses, char *original, char *line)
 {
-	char *buffer;
+	char buf[BUFFER_SIZE];
 	struct listnode *node;
 	struct listroot *root;
 
@@ -84,16 +84,15 @@ void check_all_actions(struct session *ses, char *original, char *line)
 	{
 		root->update = node->next;
 
-		if (check_one_action(line, original, node->left, ses))
+		if (check_one_regexp(ses, node, line, original, 0))
 		{
-			substitute(ses, node->right, &buffer, SUB_ARG|SUB_SEC);
+			show_debug(ses, LIST_ACTION, "#DEBUG ACTION {%s}", node->left);
 
-			show_debug(ses, LIST_ACTION, "#ACTION DEBUG: %s", buffer);
+			substitute(ses, node->right, buf, SUB_ARG|SUB_SEC);
 
-			parse_input(ses, buffer);
+			script_driver(ses, LIST_ACTION, buf);
 
 			return;
 		}
 	}
-	return;
 }
