@@ -23,6 +23,7 @@
 *               (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                  *
 *                                                                             *
 *                        coded by Peter Unold 1992                            *
+*                   recoded by Igor van den hoven 2009                        *
 ******************************************************************************/
 
 #include "tintin.h"
@@ -72,113 +73,69 @@ DO_COMMAND(do_unalias)
 	return ses;
 }
 
-int alias_regexp(char *exp, char *str)
-{
-	short cnt, arg;
-
-	while (*exp)
-	{
-#ifdef BIG5
-		if (*exp & 0x80 && *str & 0x80)
-		{
-			exp++;
-			str++;
-			if (*exp++ != *str++)
-			{
-				return FALSE;
-			}
-			continue;
-		}
-#endif
-
-		switch (exp[0])
-		{
-			case '%':
-				if (isdigit(exp[1]))
-				{
-					arg = isdigit(exp[2]) ? (exp[1] - '0') * 10 + exp[2] - '0' : exp[1] - '0';
-
-					if (exp[isdigit(exp[2]) ? 3 : 2] == 0)
-					{
-						RESTRING(gtd->vars[arg], str);
-
-						return TRUE;
-					}
-
-					for (cnt = 0 ; str[cnt] ; cnt++)
-					{
-						if (alias_regexp(exp + (isdigit(exp[2]) ? 3 : 2), &str[cnt]))
-						{
-							gtd->vars[arg] = refstring(gtd->vars[arg], "%.*s", cnt, str);
-
-							return TRUE;
-						}
-					}
-					return FALSE;
-				}
-				break;
-
-			case '^':
-				if (exp[1] == 0)
-				{
-					return (exp[1] == str[0]);
-				}
-				break;
-		}
-
-		if (*exp != *str)
-		{
-			return FALSE;
-		}
-		exp++;
-		str++;
-	}
-
-	if (*str == ' ' || *str == 0)
-	{
-		return TRUE;
-	}
-	return FALSE;
-}
-
 int check_all_aliases(struct session *ses, char *input)
 {
 	struct listnode *node;
 	struct listroot *root;
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], *arg;
+	char line[BUFFER_SIZE], tmp[BUFFER_SIZE], *arg;
 	int i;
 
 	root = ses->list[LIST_ALIAS];
 
-	arg = get_arg_stop_spaces(input, left);
-
-	RESTRING(gtd->vars[0], arg);
+	if (HAS_BIT(root->flags, LIST_FLAG_IGNORE))
+	{
+		return FALSE;
+	}
 
 	for (i = 1 ; i < 100 ; i++)
 	{
-		arg = get_arg_in_braces(arg, right, FALSE);
-
-		RESTRING(gtd->vars[i], right);
+		if (*gtd->vars[i])
+		{
+			RESTRING(gtd->vars[i], "");
+		}
 	}
 
+	substitute(ses, input, line, SUB_VAR|SUB_FUN);
+
 	for (node = root->update = root->f_node ; node ; node = root->update)
 	{
 		root->update = node->next;
 
-		if (alias_regexp(node->left, input))
+		if (check_one_regexp(ses, node, line, line, PCRE_ANCHORED))
 		{
-			substitute(ses, node->right, right, SUB_ARG);
+			i = strlen(node->left);
 
-			if (!strcmp(node->right, right) && *arg)
+			if (!strncmp(node->left, line, i))
 			{
-				sprintf(input, "%s %s", right, arg);
+				if (line[i] && line[i] != ' ')
+				{
+					continue;
+				}
+				
+				arg = get_arg_in_braces(line, tmp, FALSE);
+
+				RESTRING(gtd->vars[0], arg)
+
+				for (i = 1 ; i < 100 && *arg ; i++)
+				{
+					arg = get_arg_in_braces(arg, tmp, FALSE);
+
+					RESTRING(gtd->vars[i], tmp);
+				}
+			}
+
+			substitute(ses, node->right, tmp, SUB_ARG);
+
+			if (!strncmp(node->left, line, i) && !strcmp(node->right, tmp) && *gtd->vars[0])
+			{
+				sprintf(input, "%s %s", tmp, gtd->vars[0]);
 			}
 			else
 			{
-				sprintf(input, "%s", right);
+				sprintf(input, "%s", tmp);
 			}
 
-			show_debug(ses, LIST_ALIAS, "#ALIAS DEBUG: %s: %s", left, input);
+			show_debug(ses, LIST_ALIAS, "#DEBUG ALIAS {%s}", node->left);
 
 			DEL_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND);
 
