@@ -68,7 +68,7 @@ void init_buffer(struct session *ses, int size)
 
 	if (size)
 	{
-		ses->buffer = calloc(size, sizeof(char *));
+		ses->buffer = (char **) calloc(size, sizeof(char *));
 
 		ses->scroll_max  = size;
 		ses->scroll_row  = size - 1;
@@ -159,7 +159,7 @@ void add_line_buffer(struct session *ses, char *line, int more_output)
 	{
 		if (ses->logfile)
 		{
-			logit(ses, linebuf, ses->logfile);
+			logit(ses, linebuf, ses->logfile, TRUE);
 		}
 	}
 
@@ -170,7 +170,7 @@ void add_line_buffer(struct session *ses, char *line, int more_output)
 
 	if (ses->logline)
 	{
-		logit(ses, linebuf, ses->logline);
+		logit(ses, linebuf, ses->logline, TRUE);
 
 		fclose(ses->logline);
 		ses->logline = NULL;
@@ -198,13 +198,14 @@ void add_line_buffer(struct session *ses, char *line, int more_output)
 
 DO_COMMAND(do_grep)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], str[BUFFER_SIZE];
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
 	int scroll_cnt, grep_cnt, grep_min, grep_max, grep_add;
 
 	grep_cnt = grep_add = scroll_cnt = grep_min = 0;
 	grep_max = ses->bot_row - ses->top_row - 2;
 
 	get_arg_in_braces(arg, left, FALSE);
+	substitute(ses, left, left, SUB_VAR|SUB_FUN);
 
 	if (ses->buffer == NULL)
 	{
@@ -237,8 +238,6 @@ DO_COMMAND(do_grep)
 
 		SET_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
 
-		sprintf(str, "*%s*", right);
-
 		tintin_header(ses, " GREP %s ", right);
 
 		scroll_cnt = ses->scroll_row;
@@ -264,7 +263,7 @@ DO_COMMAND(do_grep)
 				continue;
 			}
 
-			if (regexp(str, ses->buffer[scroll_cnt], TRUE))
+			if (find(ses, ses->buffer[scroll_cnt], right))
 			{
 				grep_add = str_hash_lines(ses->buffer[scroll_cnt]);
 
@@ -303,7 +302,7 @@ DO_COMMAND(do_grep)
 				continue;
 			}
 
-			if (regexp(str, ses->buffer[scroll_cnt], TRUE))
+			if (find(ses, ses->buffer[scroll_cnt], right))
 			{
 				grep_add = str_hash_lines(ses->buffer[scroll_cnt]);
 
@@ -326,8 +325,6 @@ DO_COMMAND(do_grep)
 	return ses;
 }
 
-
-
 int show_buffer(struct session *ses)
 {
 	char temp[STRING_SIZE], wrap[STRING_SIZE], *temp_ptr, *wrap_ptr;
@@ -418,13 +415,13 @@ int show_buffer(struct session *ses)
 					}
 					continue;
 
-				case '\r':
+				case '\n':
 					temp_ptr++;
 					scroll_tmp--;
 					continue;
 
 				case '\0':
-					printf("\033[1;31m(buffer error)\n\r");
+					printf("\033[1;31m(buffer error)\n");
 					scroll_tmp--;
 					break;
 
@@ -437,7 +434,7 @@ int show_buffer(struct session *ses)
 
 		if (scroll_add >= 0)
 		{
-			printf("%s%s\n\r", wrap, temp_ptr);
+			printf("%s%s\n", wrap, temp_ptr);
 		}
 		else
 		{
@@ -445,10 +442,10 @@ int show_buffer(struct session *ses)
 
 			for (wrap_ptr = temp_ptr ; scroll_tmp++ < scroll_size ; temp_ptr++)
 			{
-				temp_ptr = strchr(temp_ptr, '\r');
+				temp_ptr = strchr(temp_ptr, '\n');
 			}
 			*temp_ptr = 0;
-			printf("%s", wrap_ptr);
+			printf("%s\n", wrap_ptr);
 
 			goto eof;
 		}
@@ -484,11 +481,11 @@ int show_buffer(struct session *ses)
 
 		scroll_add -= scroll_tmp;
 
-		printf("%s\n\r", temp);
+		printf("%s\n", temp);
 	}
 
 	/*
-		If the bottom line exits of multiple lines split in the middle
+		If the bottom line exists of multiple lines split in the middle
 	*/
 
 	if (scroll_tmp && ses->buffer[scroll_cnt])
@@ -497,12 +494,11 @@ int show_buffer(struct session *ses)
 
 		while (scroll_tmp--)
 		{
-			temp_ptr = strchr(temp_ptr, '\r');
-			temp_ptr++;
+			temp_ptr = strchr(temp_ptr, '\n');
 		}
 		*temp_ptr = 0;
 
-		printf("%s", temp);
+		printf("%s\n", temp);
 	}
 
 	eof:
@@ -519,90 +515,80 @@ int show_buffer(struct session *ses)
 DO_COMMAND(do_buffer)
 {
 	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+	int cnt;
 
 	arg = get_arg_in_braces(arg, left, FALSE);
 	arg = get_arg_in_braces(arg, right, TRUE);
+	substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
-	switch (left[0])
+	if (*left == 0)
 	{
-		case 'h':
-			buffer_h("");
-			break;
+		tintin_header(ses, " BUFFER COMMANDS ");
 
-		case 'u':
-			buffer_u("");
-			break;
+		for (cnt = 0 ; *buffer_table[cnt].name != 0 ; cnt++)
+		{
+			tintin_printf2(ses, "  [%-13s] %s", buffer_table[cnt].name, buffer_table[cnt].desc);
+		}
+		tintin_header(ses, "");
 
-		case 'd':
-			buffer_d("");
-			break;
-
-		case 'e':
-			buffer_e("");
-			break;
-
-		case 'f':
-			buffer_f(right);
-			break;
-
-		case 'w':
-			do_writebuffer(ses, right);
-			break;
-
-		case 'i':
-			do_hash(ses, right);
-			break;
-
-		case 'l':
-			buffer_l("");
-			break;
-
-		default:
-			buffer_e("");
-			break;
+		return ses;
 	}
 
+	for (cnt = 0 ; *buffer_table[cnt].name ; cnt++)
+	{
+		if (!is_abbrev(left, buffer_table[cnt].name))
+		{
+			continue;
+		}
+
+		buffer_table[cnt].fun(ses, right);
+
+		return ses;
+	}
+
+	do_buffer(ses, "");
+
 	return ses;
 }
 
-DO_CURSOR(buffer_u)
+DO_BUFFER(buffer_up)
 {
 	int scroll_size, scroll_cnt, buffer_add, buffer_tmp;
 
-	if (gtd->ses->buffer == NULL)
+	if (ses->buffer == NULL)
 	{
 		return;
 	}
 
-	if (gtd->ses->scroll_line == -1)
+	if (ses->scroll_line == -1)
 	{
-		gtd->ses->scroll_line = gtd->ses->scroll_row + 1;
+		ses->scroll_line = ses->scroll_row + 1;
 	}
 
-	scroll_size = get_scroll_size(gtd->ses);
-	scroll_cnt  = gtd->ses->scroll_line;
-	buffer_add  = 0 - gtd->ses->scroll_base;
+	scroll_size = get_scroll_size(ses);
+	scroll_cnt  = ses->scroll_line;
+	buffer_add  = 0 - ses->scroll_base;
 
 	while (TRUE)
 	{
-		if (gtd->ses->buffer[scroll_cnt] == NULL)
+		if (ses->buffer[scroll_cnt] == NULL)
 		{
-			break;
+			return;
 		}
 
-		buffer_tmp = str_hash_lines(gtd->ses->buffer[scroll_cnt]);
+		buffer_tmp = str_hash_lines(ses->buffer[scroll_cnt]);
 
 		if (scroll_size < buffer_add + buffer_tmp)
 		{
-			gtd->ses->scroll_line = scroll_cnt;
-			gtd->ses->scroll_base = scroll_size - buffer_add;
+			ses->scroll_line = scroll_cnt;
+			ses->scroll_base = scroll_size - buffer_add;
 
 			break;
 		}
 
 		buffer_add += buffer_tmp;
 
-		if (scroll_cnt == gtd->ses->scroll_max - 1)
+		if (scroll_cnt == ses->scroll_max - 1)
 		{
 			scroll_cnt = 0;
 		}
@@ -612,38 +598,38 @@ DO_CURSOR(buffer_u)
 		}
 	}
 
-	show_buffer(gtd->ses);
+	show_buffer(ses);
 }
 
-DO_CURSOR(buffer_d)
+DO_BUFFER(buffer_down)
 {
 	int scroll_size, scroll_cnt, buffer_add, buffer_tmp;
 
-	if (gtd->ses->buffer == NULL)
+	if (ses->buffer == NULL)
 	{
 		return;
 	}
 
-	if (gtd->ses->scroll_line == -1)
+	if (ses->scroll_line == -1)
 	{
 		return;
 	}
 
-	scroll_size = get_scroll_size(gtd->ses);
-	scroll_cnt  = gtd->ses->scroll_line;
-	buffer_add  = gtd->ses->scroll_base;
+	scroll_size = get_scroll_size(ses);
+	scroll_cnt  = ses->scroll_line;
+	buffer_add  = ses->scroll_base;
 
 	if (scroll_size <= buffer_add)
 	{
-		gtd->ses->scroll_base = buffer_add - scroll_size;
+		ses->scroll_base = buffer_add - scroll_size;
 
-		show_buffer(gtd->ses);
+		show_buffer(ses);
 		return;
 	}
 
 	if (scroll_cnt == 0)
 	{
-		scroll_cnt = gtd->ses->scroll_max - 1;
+		scroll_cnt = ses->scroll_max - 1;
 	}
 	else
 	{
@@ -652,18 +638,18 @@ DO_CURSOR(buffer_d)
 
 	while (TRUE)
 	{
-		if (gtd->ses->buffer[scroll_cnt] == NULL)
+		if (ses->buffer[scroll_cnt] == NULL)
 		{
-			buffer_e("");
+			buffer_end(ses, "");
 			return;
 		}
 
-		buffer_tmp = str_hash_lines(gtd->ses->buffer[scroll_cnt]);
+		buffer_tmp = str_hash_lines(ses->buffer[scroll_cnt]);
 
 		if (scroll_size <= buffer_add + buffer_tmp)
 		{
-			gtd->ses->scroll_line = scroll_cnt;
-			gtd->ses->scroll_base = buffer_tmp - (scroll_size - buffer_add);
+			ses->scroll_line = scroll_cnt;
+			ses->scroll_base = buffer_tmp - (scroll_size - buffer_add);
 
 			break;
 		}
@@ -672,7 +658,7 @@ DO_CURSOR(buffer_d)
 
 		if (scroll_cnt == 0)
 		{
-			scroll_cnt = gtd->ses->scroll_max - 1;
+			scroll_cnt = ses->scroll_max - 1;
 		}
 		else
 		{
@@ -680,80 +666,80 @@ DO_CURSOR(buffer_d)
 		}
 	}
 
-	show_buffer(gtd->ses);
+	show_buffer(ses);
 }
 
-DO_CURSOR(buffer_h)
+DO_BUFFER(buffer_home)
 {
-	if (gtd->ses->buffer == NULL)
+	if (ses->buffer == NULL)
 	{
 		return;
 	}
 
-	if (gtd->ses->buffer[0])
+	if (ses->buffer[0])
 	{
-		gtd->ses->scroll_line = gtd->ses->scroll_row - 1;
+		ses->scroll_line = ses->scroll_row - 1;
 	}
 	else
 	{
-		gtd->ses->scroll_line = gtd->ses->scroll_max - 1;
+		ses->scroll_line = ses->scroll_max - 1;
 	}
-	gtd->ses->scroll_base = str_hash_lines(gtd->ses->buffer[gtd->ses->scroll_line]);
+	ses->scroll_base = str_hash_lines(ses->buffer[ses->scroll_line]);
 
-	buffer_d("");
+	buffer_down(ses, "");
 }
 
-DO_CURSOR(buffer_e)
+DO_BUFFER(buffer_end)
 {
-	if (gtd->ses->buffer == NULL)
+	if (ses->buffer == NULL)
 	{
 		return;
 	}
 
-	if (gtd->ses->scroll_row == gtd->ses->scroll_max - 1)
+	if (ses->scroll_row == ses->scroll_max - 1)
 	{
-		gtd->ses->scroll_line = 0;
+		ses->scroll_line = 0;
 	}
 	else
 	{
-		gtd->ses->scroll_line = gtd->ses->scroll_row + 1;
+		ses->scroll_line = ses->scroll_row + 1;
 	}
 
-	gtd->ses->scroll_base = 0;
+	ses->scroll_base = 0;
 
-	show_buffer(gtd->ses);
+	show_buffer(ses);
 
-	gtd->ses->scroll_line = -1;
-	gtd->ses->scroll_base =  0;
+	ses->scroll_line = -1;
+	ses->scroll_base =  0;
 }
 
-DO_CURSOR(buffer_l)
+DO_BUFFER(buffer_lock)
 {
-	if (gtd->ses->buffer == NULL)
+	if (ses->buffer == NULL)
 	{
 		return;
 	}
 
-	if (gtd->ses->scroll_line == -1)
+	if (ses->scroll_line == -1)
 	{
-		gtd->ses->scroll_line = gtd->ses->scroll_row + 1;
+		ses->scroll_line = ses->scroll_row + 1;
 	}
 	else
 	{
-		buffer_e("");
+		buffer_end(ses, "");
 	}
 }
 
-void buffer_f(char *arg)
+DO_BUFFER(buffer_find)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], str[BUFFER_SIZE];
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
 	int scroll_cnt, grep_cnt, grep_max;
 
 	grep_cnt = grep_max = scroll_cnt = 0;
 
 	get_arg_in_braces(arg, left, FALSE);
 
-	if (gtd->ses->buffer == NULL)
+	if (ses->buffer == NULL)
 	{
 		tintin_puts2(NULL, "#BUFFER, NO SCROLL BUFFER AVAILABLE.");
 	}
@@ -781,15 +767,13 @@ void buffer_f(char *arg)
 			arg = get_arg_in_braces(arg, right, TRUE);
 		}
 
-		sprintf(str, "*%s*", right);
-
 		if (grep_max >= 0)
 		{
-			scroll_cnt = gtd->ses->scroll_row;
+			scroll_cnt = ses->scroll_row;
 
 			do
 			{
-				if (scroll_cnt == gtd->ses->scroll_max -1)
+				if (scroll_cnt == ses->scroll_max -1)
 				{
 					scroll_cnt = 0;
 				}
@@ -798,17 +782,17 @@ void buffer_f(char *arg)
 					scroll_cnt++;
 				}
 
-				if (gtd->ses->buffer[scroll_cnt] == NULL)
+				if (ses->buffer[scroll_cnt] == NULL)
 				{
 					break;
 				}
 
-				if (str_hash_grep(gtd->ses->buffer[scroll_cnt], FALSE))
+				if (str_hash_grep(ses->buffer[scroll_cnt], FALSE))
 				{
 					continue;
 				}
 
-				if (regexp(str, gtd->ses->buffer[scroll_cnt], TRUE))
+				if (find(ses, ses->buffer[scroll_cnt], right))
 				{
 					if (grep_cnt == grep_max)
 					{
@@ -817,41 +801,41 @@ void buffer_f(char *arg)
 					grep_cnt++;
 				}
 			}
-			while (scroll_cnt != gtd->ses->scroll_row);
+			while (scroll_cnt != ses->scroll_row);
 		}
 		else
 		{
-			if (gtd->ses->buffer[0])
+			if (ses->buffer[0])
 			{
-				scroll_cnt = gtd->ses->scroll_row - 1;
+				scroll_cnt = ses->scroll_row - 1;
 			}
 			else
 			{
-				scroll_cnt = gtd->ses->scroll_max - 1;
+				scroll_cnt = ses->scroll_max - 1;
 			}
 
 			do
 			{
 				if (scroll_cnt == 0)
 				{
-					scroll_cnt = gtd->ses->scroll_max -1;
+					scroll_cnt = ses->scroll_max -1;
 				}
 				else
 				{
 					scroll_cnt--;
 				}
 
-				if (gtd->ses->buffer[scroll_cnt] == NULL)
+				if (ses->buffer[scroll_cnt] == NULL)
 				{
 					break;
 				}
 
-				if (str_hash_grep(gtd->ses->buffer[scroll_cnt], FALSE))
+				if (str_hash_grep(ses->buffer[scroll_cnt], FALSE))
 				{
 					continue;
 				}
 
-				if (regexp(str, gtd->ses->buffer[scroll_cnt], TRUE))
+				if (find(ses, ses->buffer[scroll_cnt], right))
 				{
 					grep_cnt--;
 
@@ -862,18 +846,87 @@ void buffer_f(char *arg)
 
 				}
 			}
-			while (scroll_cnt != gtd->ses->scroll_row);
+			while (scroll_cnt != ses->scroll_row);
 		}
 
-		if (gtd->ses->buffer[scroll_cnt] == NULL || scroll_cnt == gtd->ses->scroll_row)
+		if (ses->buffer[scroll_cnt] == NULL || scroll_cnt == ses->scroll_row)
 		{
 			tintin_puts(NULL, "#BUFFER FIND, NO MATCHES FOUND.");
 		}
 		else
 		{
-			gtd->ses->scroll_line = scroll_cnt;
+			ses->scroll_line = scroll_cnt;
 
-			show_buffer(gtd->ses);
+			show_buffer(ses);
+		}
+	}
+	return;
+}
+
+DO_BUFFER(buffer_write)
+{
+	FILE *fp;
+	char left[BUFFER_SIZE], out[STRING_SIZE];
+	int cnt;
+
+	arg = get_arg_in_braces(arg, left, FALSE);
+
+	if (*left == 0)
+	{
+		tintin_printf(ses, "#SYNTAX: #BUFFER WRITE <FILENAME>]");
+	}
+	else
+	{
+		if ((fp = fopen(left, "w")))
+		{
+			show_message(ses, LIST_MESSAGE, "#OK: WRITING BUFFER TO '%s'.", left);
+
+			if (HAS_BIT(ses->flags, SES_FLAG_LOGHTML))
+			{
+				write_html_header(fp);
+			}
+
+			cnt = ses->scroll_row;
+
+			do
+			{
+				if (cnt == 0)
+				{
+					cnt = ses->scroll_max - 1;
+				}
+				else
+				{
+					cnt--;
+				}
+
+				if (ses->buffer[cnt] == NULL)
+				{
+					continue;
+				}
+
+				if (HAS_BIT(ses->flags, SES_FLAG_LOGPLAIN))
+				{
+					strip_vt102_codes(ses->buffer[cnt], out);
+				}
+				else if (HAS_BIT(ses->flags, SES_FLAG_LOGHTML))
+				{
+					vt102_to_html(ses, ses->buffer[cnt], out);
+				}
+				else
+				{
+					strcpy(out, ses->buffer[cnt]);
+				}
+				strcat(out, "\n");
+
+				fputs(out, fp);
+			}
+			while (cnt != ses->scroll_row);
+
+			fclose(fp);
+		}
+		else
+		{
+			tintin_printf(ses, "#ERROR: #BUFFER WRITE {%s} - FAILED TO OPEN FILE.", left);
 		}
 	}
 	return;
