@@ -31,7 +31,7 @@
 
 DO_COMMAND(do_class)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], pr[BUFFER_SIZE];
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], third[BUFFER_SIZE];
 
 	struct listroot *root;
 	struct listnode *node;
@@ -41,7 +41,8 @@ DO_COMMAND(do_class)
 
 	arg = get_arg_in_braces(arg, left,  FALSE);
 	arg = get_arg_in_braces(arg, right, FALSE);
-	arg = get_arg_in_braces(arg, pr,    FALSE);
+	arg = get_arg_in_braces(arg, third, FALSE);
+	substitute(ses, third, third, SUB_VAR|SUB_FUN);
 
 	if (*left == 0)
 	{
@@ -49,12 +50,12 @@ DO_COMMAND(do_class)
 
 		for (node = root->f_node ; node ; node = node->next)
 		{
-			tintin_printf2(ses, "%-20s %3d %s", node->left, count_class(ses, node), !strcmp(ses->class, node->left) ? "OPEN" : "CLOSED");
+			tintin_printf2(ses, "%-20s %3d %s", node->left, count_class(ses, node), !strcmp(ses->group, node->left) ? "OPEN" : "CLOSED");
 		}
 	}
 	else if (*right == 0)
 	{
-		if (search_node_with_wild(root, left))
+		if (search_node_with_wild(ses, left, LIST_CLASS))
 		{
 			tintin_header(ses, " %s ", left);
 
@@ -62,7 +63,7 @@ DO_COMMAND(do_class)
 			{
 				for (node = ses->list[cnt]->f_node ; node ; node = node->next)
 				{
-					if (!strcmp(node->class, left))
+					if (!strcmp(node->group, left))
 					{
 						shownode_list(ses, node, cnt);
 					}
@@ -93,16 +94,16 @@ DO_COMMAND(do_class)
 		{
 			if (!searchnode_list(root, left))
 			{
-				updatenode_list(ses, left, right, pr, LIST_CLASS);
+				updatenode_list(ses, left, right, third, LIST_CLASS);
 			}
-			class_table[cnt].class(ses, left, pr);
+			class_table[cnt].group(ses, left, third);
 		}
 	}
 	return ses;
 }
 
 
-int count_class(struct session *ses, struct listnode *class)
+int count_class(struct session *ses, struct listnode *group)
 {
 	struct listnode *node;
 	int list, cnt;
@@ -116,7 +117,7 @@ int count_class(struct session *ses, struct listnode *class)
 
 		for (node = ses->list[list]->f_node ; node ; node = node->next)
 		{
-			if (!strcmp(node->class, class->left))
+			if (!strcmp(node->group, group->left))
 			{
 				cnt++;
 			}
@@ -130,9 +131,9 @@ DO_CLASS(class_open)
 {
 	struct listnode *node;
 
-	node = search_node_with_wild(ses->list[LIST_CLASS], left);
+	node = search_node_with_wild(ses, left, LIST_CLASS);
 
-	RESTRING(ses->class, left);
+	RESTRING(ses->group, left);
 
 	show_message(ses, LIST_CLASS, "#CLASS {%s} HAS BEEN OPENED.", left);
 
@@ -144,11 +145,11 @@ DO_CLASS(class_close)
 {
 	struct listnode *node;
 
-	node = search_node_with_wild(ses->list[LIST_CLASS], left);
+	node = search_node_with_wild(ses, left, LIST_CLASS);
 
-	if (!strcmp(ses->class, left))
+	if (!strcmp(ses->group, left))
 	{
-		RESTRING(ses->class, "");
+		RESTRING(ses->group, "");
 
 		show_message(ses, LIST_CLASS, "#CLASS {%s} HAS BEEN CLOSED.", left);
 	}
@@ -197,7 +198,7 @@ DO_CLASS(class_write)
 
 		for (node = ses->list[cnt]->f_node ; node ; node = node->next)
 		{
-			if (!strcmp(node->class, left))
+			if (!strcmp(node->group, left))
 			{
 				prepare_for_write(cnt, node, temp);
 
@@ -217,14 +218,14 @@ DO_CLASS(class_write)
 
 DO_CLASS(class_kill)
 {
-	struct listnode *node, *node_next, *class;
+	struct listnode *node, *node_next, *group;
 	int cnt;
 
-	class = search_node_with_wild(ses->list[LIST_CLASS], left);
+	group = search_node_with_wild(ses, left, LIST_CLASS);
 
-	if (!strcmp(ses->class, left))
+	if (!strcmp(ses->group, left))
 	{
-		RESTRING(ses->class, "");
+		RESTRING(ses->group, "");
 	}
 
 	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
@@ -238,7 +239,7 @@ DO_CLASS(class_kill)
 		{
 			node_next = node->next;
 
-			if (!strcmp(node->class, left))
+			if (!strcmp(node->group, left))
 			{
 				deletenode_list(ses, node, cnt);
 			}
@@ -246,7 +247,7 @@ DO_CLASS(class_kill)
 	}
 	show_message(ses, LIST_CLASS, "#CLASS {%s} HAS BEEN CLEARED.", left);
 
-	deletenode_list(ses, class, LIST_CLASS);
+	deletenode_list(ses, group, LIST_CLASS);
 
 	return ses;
 }
