@@ -40,6 +40,7 @@ DO_COMMAND(do_configure)
 
 	arg = get_arg_in_braces(arg, left,  FALSE);
 	arg = get_arg_in_braces(arg, right, FALSE);
+	substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
 	if (*left == 0)
 	{
@@ -49,7 +50,7 @@ DO_COMMAND(do_configure)
 
 		for (node = root->f_node ; node ; node = node->next)
 		{
-			tintin_printf2(ses, "  [%-13s] [%6s] %s", 
+			tintin_printf2(ses, "[%-13s] [%8s] %s", 
 				node->left,
 				node->right,
 				strcmp(node->right, "ON") == 0 ? config_table[atoi(node->pr)].msg_on : config_table[atoi(node->pr)].msg_off);
@@ -152,7 +153,28 @@ DO_CONFIG(config_repeatenter)
 	return ses;
 }
 
-DO_CONFIG(config_echocommand)
+DO_CONFIG(config_commandcolor)
+{
+	char str[BUFFER_SIZE];
+
+	if (!is_color_code(arg))
+	{
+		tintin_printf2(ses, "#CONFIG COMMAND COLOR: Invalid color code '%s'.", arg);
+
+		return NULL;
+	}
+	substitute(ses, arg, str, SUB_COL);
+
+	RESTRING(ses->cmd_color, str);
+
+	sprintf(str, "%d", index);
+
+	updatenode_list(ses, config_table[index].name, arg, str, LIST_CONFIG);
+
+	return ses;
+}
+
+DO_CONFIG(config_commandecho)
 {
 	char str[BUFFER_SIZE];
 
@@ -342,14 +364,14 @@ DO_CONFIG(config_packetpatch)
 		return NULL;
 	}
 
-	if (atoi(arg) < 0 || atoi(arg) > 10000)
+	if (atof(arg) < 0 || atoi(arg) > 10)
 	{
-		tintin_printf(ses, "#ERROR: #CONFIG PACKET PATCH: PROVIDE A NUMBER BETWEEN 0 and 10000");
+		tintin_printf(ses, "#ERROR: #CONFIG PACKET PATCH: PROVIDE A NUMBER BETWEEN 0.00 and 10.00");
 
 		return NULL;
 	}
 
-	gts->check_output = atoi(arg) * 1000;
+	gts->check_output = (long long) (get_number(ses, arg) * 1000000LL);
 
 	sprintf(str, "%d", index);
 
@@ -522,3 +544,80 @@ DO_CONFIG(config_colorpatch)
 
 	return ses;
 }
+
+DO_CONFIG(config_regexp)
+{
+	char str[BUFFER_SIZE];
+
+	if (!strcasecmp(arg, "ON"))
+	{
+		SET_BIT(ses->flags, SES_FLAG_REGEXP);
+	}
+	else if (!strcasecmp(arg, "OFF"))
+	{
+		DEL_BIT(ses->flags, SES_FLAG_REGEXP);
+	}
+	else
+	{
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+
+		return NULL;
+	}
+	sprintf(str, "%d", index);
+
+	updatenode_list(ses, config_table[index].name, capitalize(arg), str, LIST_CONFIG);
+
+	return ses;
+}
+
+DO_CONFIG(config_mccp)
+{
+	char str[BUFFER_SIZE];
+
+	if (!strcasecmp(arg, "ON"))
+	{
+		SET_BIT(ses->flags, SES_FLAG_MCCP);
+	}
+	else if (!strcasecmp(arg, "OFF"))
+	{
+		DEL_BIT(ses->flags, SES_FLAG_MCCP);
+	}
+	else
+	{
+		tintin_printf(ses, "#SYNTAX: #CONFIG {%s} <ON|OFF>", config_table[index].name);
+
+		return NULL;
+	}
+	sprintf(str, "%d", index);
+
+	updatenode_list(ses, config_table[index].name, capitalize(arg), str, LIST_CONFIG);
+
+	return ses;
+}
+
+DO_CONFIG(config_autotab)
+{
+	char str[BUFFER_SIZE];
+
+	if (!is_number(arg))
+	{
+		tintin_printf(ses, "#SYNTAX: #CONFIG {AUTO TAB} <NUMBER>");
+
+		return NULL;
+	}
+
+	if (atoi(arg) < 1 || atoi(arg) > 999999)
+	{
+		tintin_printf(ses, "#ERROR: #CONFIG BUFFER: PROVIDE A NUMBER BETWEEN 1 and 999999");
+
+		return NULL;
+	}
+
+	ses->auto_tab = atoi(arg);
+
+	sprintf(str, "%d", index);
+
+	updatenode_list(ses, config_table[index].name, capitalize(arg), str, LIST_CONFIG);
+
+	return ses;
+}
