@@ -20,8 +20,8 @@
 *******************************************************************************/
 
 /******************************************************************************
-*   file: cursor.c - funtions related to line editing                         *
 *              (T)he K(I)cki(N) (T)ickin D(I)kumud Clie(N)t                   *
+*                                                                             *
 *                     coded by Igor van den Hoven 2006                        *
 ******************************************************************************/
 
@@ -35,6 +35,7 @@ DO_COMMAND(do_cursor)
 
 	arg = get_arg_in_braces(arg, left,  TRUE);
 	arg = get_arg_in_braces(arg, right, TRUE);
+	substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
 	if (*left == 0)
 	{
@@ -82,6 +83,33 @@ DO_CURSOR(cursor_backspace)
 	cursor_delete("");
 }
 
+
+DO_CURSOR(cursor_buffer_down)
+{
+	buffer_down(gtd->ses, "");
+}
+
+DO_CURSOR(cursor_buffer_end)
+{
+	buffer_end(gtd->ses, "");
+}
+
+DO_CURSOR(cursor_buffer_home)
+{
+	buffer_home(gtd->ses, "");
+}
+
+DO_CURSOR(cursor_buffer_lock)
+{
+	buffer_lock(gtd->ses, "");
+}
+
+DO_CURSOR(cursor_buffer_up)
+{
+	buffer_up(gtd->ses, "");
+}
+
+
 DO_CURSOR(cursor_check_line)
 {
 	if (gtd->input_pos != gtd->input_cur)
@@ -142,6 +170,11 @@ DO_CURSOR(cursor_clear_left)
 		gtd->input_cur  = 0;
 		gtd->input_pos  = 0;
 	}
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
+	{
+		cursor_history_find("");
+	}
 }
 
 DO_CURSOR(cursor_clear_line)
@@ -153,7 +186,7 @@ DO_CURSOR(cursor_clear_line)
 
 	if (gtd->input_pos)
 	{
-		input_printf("\033[%dD\033[%dP", gtd->input_pos, gtd->input_len);
+		input_printf("\033[%dG\033[%dP", gtd->input_off, gtd->input_len);
 	}
 	else
 	{
@@ -164,6 +197,11 @@ DO_CURSOR(cursor_clear_line)
 	gtd->input_cur = 0;
 	gtd->input_pos = 0;
 	gtd->input_buf[0] = 0;
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
+	{
+		cursor_history_find("");
+	}
 }
 
 DO_CURSOR(cursor_clear_right)
@@ -180,13 +218,31 @@ DO_CURSOR(cursor_clear_right)
 	gtd->input_buf[gtd->input_cur] = 0;
 
 	gtd->input_len = gtd->input_cur;
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
+	{
+		cursor_history_find("");
+	}
 }
 
 DO_CURSOR(cursor_convert_meta)
 {
 	SET_BIT(gtd->flags, TINTIN_FLAG_CONVERTMETACHAR);
 }
-	
+
+
+DO_CURSOR(cursor_delete_exit)
+{
+	if (gtd->input_len == 0)
+	{
+		cursor_exit("");
+	}
+	else
+	{
+		cursor_delete("");
+	}
+}
+
 DO_CURSOR(cursor_delete)
 {
 	if (gtd->input_len == 0)
@@ -206,12 +262,16 @@ DO_CURSOR(cursor_delete)
 	input_printf("\033[1P");
 
 	cursor_fix_line("");
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
+	{
+		cursor_history_find("");
+	}
 }
 
-DO_CURSOR(cursor_delete_word)
+DO_CURSOR(cursor_delete_word_left)
 {
 	int index_cur;
-	int index_pos;
 
 	if (gtd->input_cur == 0)
 	{
@@ -219,41 +279,77 @@ DO_CURSOR(cursor_delete_word)
 	}
 
 	index_cur = gtd->input_cur;
-	index_pos = gtd->input_pos;
 
-	gtd->input_cur--;
-	gtd->input_pos--;
-
-	while (gtd->input_cur && gtd->input_buf[gtd->input_cur] == ' ')
+	while (gtd->input_cur && gtd->input_buf[gtd->input_cur - 1] == ' ')
 	{
 		gtd->input_cur--;
 		gtd->input_pos--;
 	}
 
-	if (gtd->input_cur)
+	while (gtd->input_cur && gtd->input_buf[gtd->input_cur - 1] != ' ')
 	{
-		while (gtd->input_cur && gtd->input_buf[gtd->input_cur] != ' ')
-		{
-			gtd->input_cur--;
-			gtd->input_pos--;
-		}
-
-		if (gtd->input_buf[gtd->input_cur] == ' ')
-		{
-			gtd->input_cur++;
-			gtd->input_pos++;
-		}
+		gtd->input_cur--;
+		gtd->input_pos--;
 	}
 
 	sprintf(gtd->paste_buf, "%.*s", index_cur - gtd->input_cur, &gtd->input_buf[gtd->input_cur]);
 
-	memmove(&gtd->input_buf[gtd->input_cur], &gtd->input_buf[index_cur], index_cur + 1);
+	memmove(&gtd->input_buf[gtd->input_cur], &gtd->input_buf[index_cur], gtd->input_len - index_cur + 1);
 
-	input_printf("\033[%dD\033[%dP", index_pos - gtd->input_pos, index_pos - gtd->input_pos);
+	input_printf("\033[%dD\033[%dP", index_cur - gtd->input_cur, index_cur - gtd->input_cur);
 
 	gtd->input_len -= index_cur - gtd->input_cur;
 
 	cursor_check_line("");
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
+	{
+		cursor_history_find("");
+	}
+}
+
+
+DO_CURSOR(cursor_delete_word_right)
+{
+	int index_cur, index_pos;
+
+	if (gtd->input_cur == gtd->input_len)
+	{
+		return;
+	}
+
+	index_cur = gtd->input_cur;
+	index_pos = gtd->input_pos;
+
+	while (gtd->input_cur != gtd->input_len && gtd->input_buf[gtd->input_cur] == ' ')
+	{
+		gtd->input_cur++;
+		gtd->input_pos++;
+	}
+
+	while (gtd->input_cur != gtd->input_len && gtd->input_buf[gtd->input_cur] != ' ')
+	{
+		gtd->input_cur++;
+		gtd->input_pos++;
+	}
+
+	sprintf(gtd->paste_buf, "%.*s", gtd->input_cur - index_cur, &gtd->input_buf[gtd->input_cur]);
+
+	memmove(&gtd->input_buf[index_cur], &gtd->input_buf[gtd->input_cur], gtd->input_len - gtd->input_cur + 1);
+
+	input_printf("\033[%dP", gtd->input_cur - index_cur);
+
+	gtd->input_len -= gtd->input_cur - index_cur;
+
+	gtd->input_pos = index_pos;
+	gtd->input_cur = index_cur;
+
+	cursor_fix_line("");
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
+	{
+		cursor_history_find("");
+	}
 }
 
 DO_CURSOR(cursor_echo)
@@ -272,7 +368,7 @@ DO_CURSOR(cursor_echo)
 	}
 	else
 	{
-		tintin_printf(NULL, "#SYNTAX: #CURSOR {ECHO} {ON|OFF}");
+		tintin_printf(NULL, "#SYNTAX: #CURSOR {ECHO} {ON|OFF}.");
 	}
 }
 
@@ -287,9 +383,44 @@ DO_CURSOR(cursor_end)
 	cursor_redraw_line("");
 }
 
+DO_CURSOR(cursor_enter)
+{
+	input_printf("\n");
+
+	gtd->input_buf[gtd->input_len] = 0;
+
+	gtd->input_len    = 0;
+	gtd->input_cur    = 0;
+	gtd->input_pos    = 0;
+	gtd->macro_buf[0] = 0;
+	gtd->input_tmp[0] = 0;
+
+	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
+	{
+		DEL_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH);
+
+		if (gtd->input_his)
+		{
+			strcpy(gtd->input_buf, gtd->input_his->left);
+		}
+
+		gtd->input_off = 1;
+	}
+	gtd->input_his = NULL;
+
+	SET_BIT(gtd->flags, TINTIN_FLAG_PROCESSINPUT);
+}
+
 DO_CURSOR(cursor_exit)
 {
-	do_zap(gtd->ses, "");
+	if (gtd->ses == gts)
+	{
+		do_end(NULL, "");
+	}
+	else
+	{
+		cleanup_session(gtd->ses);
+	}
 }
 
 DO_CURSOR(cursor_history_next)
@@ -305,7 +436,7 @@ DO_CURSOR(cursor_history_next)
 
 		for (node = gtd->input_his->next ; node ; node = node->next)
 		{
-			if (*gtd->input_buf && strstr(node->left, gtd->input_buf))
+			if (*gtd->input_buf && find(gtd->ses, node->left, gtd->input_buf))
 			{
 				break;
 			}
@@ -313,7 +444,7 @@ DO_CURSOR(cursor_history_next)
 
 		if (node)
 		{
-			input_printf("\0337\033[%dC\033[0K%.*s\0338", gtd->input_len - gtd->input_cur + 3, gtd->ses->cols - 16 - gtd->input_len, node->left);
+			input_printf("\033[%dC   \033[0K%.*s\033[%dG", gtd->input_len - gtd->input_cur + 1, gtd->ses->cols - 16 - gtd->input_len, node->left, gtd->input_pos + 11);
 
 			gtd->input_his = node;
 		}
@@ -358,14 +489,14 @@ DO_CURSOR(cursor_history_prev)
 
 	if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
 	{
-		if (gtd->input_his == NULL)
+		if (gtd->input_his == NULL || *gtd->input_buf == 0)
 		{
 			return;
 		}
 
 		for (node = gtd->input_his->prev ; node ; node = node->prev)
 		{
-			if (*gtd->input_buf && strstr(node->left, gtd->input_buf))
+			if (find(gtd->ses, node->left, gtd->input_buf))
 			{
 				break;
 			}
@@ -373,7 +504,7 @@ DO_CURSOR(cursor_history_prev)
 
 		if (node)
 		{
-			input_printf("\0337\033[%dC\033[0K%.*s\0338", gtd->input_len - gtd->input_cur + 3, gtd->ses->cols - 16 - gtd->input_len, node->left);
+			input_printf("\033[%dC   \033[0K%.*s\033[%dG", gtd->input_len - gtd->input_cur + 1, gtd->ses->cols - 16 - gtd->input_len, node->left, gtd->input_pos + 11);
 
 			gtd->input_his = node;
 		}
@@ -433,29 +564,54 @@ DO_CURSOR(cursor_history_search)
 
 	if (!HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
 	{
-		SET_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH);
-
 		strcpy(gtd->input_tmp, gtd->input_buf);
 
 		cursor_clear_line("");
 
+		SET_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH);
+
+		gtd->input_off = 11;
+
 		input_printf("(search) [ ] \033[3D");
 	}
 	else
 	{
-		strcpy(gtd->input_buf, gtd->input_tmp);
+		if (gtd->input_his)
+		{
+			strcpy(gtd->input_buf, gtd->input_his->left);
 
-		input_printf("\033[%dD\033[0K", 10 + gtd->input_pos);
+			input_printf("\033[1G\033[0K");
 
-		gtd->input_len = strlen(gtd->input_buf);
-		gtd->input_cur = gtd->input_len;
-		gtd->input_pos = gtd->input_len;
+			gtd->input_len = strlen(gtd->input_buf);
+			gtd->input_cur = gtd->input_len;
+			gtd->input_pos = gtd->input_len;
 
-		gtd->input_his = NULL;
+			gtd->input_his = NULL;
 
-		DEL_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH);
+			DEL_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH);
 
-		cursor_redraw_line("");
+			gtd->input_off = 1;
+
+			cursor_redraw_line("");
+		}
+		else
+		{
+			strcpy(gtd->input_buf, gtd->input_tmp);
+
+			input_printf("\033[1G\033[0K");
+
+			gtd->input_len = strlen(gtd->input_buf);
+			gtd->input_cur = gtd->input_len;
+			gtd->input_pos = gtd->input_len;
+
+			gtd->input_his = NULL;
+
+			DEL_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH);
+
+			gtd->input_off = 1;
+
+			cursor_redraw_line("");
+		}
 	}
 }
 
@@ -470,20 +626,35 @@ DO_CURSOR(cursor_history_find)
 
 	for (node = root->l_node ; node ; node = node->prev)
 	{
-		if (*gtd->input_buf && strstr(node->left, gtd->input_buf))
+		if (*gtd->input_buf && find(gtd->ses, node->left, gtd->input_buf))
 		{
 			break;
 		}
 	}
+
 	gtd->input_his = node;
 
 	if (node)
 	{
-		input_printf("\0337\033[%dC\033[0K%.*s\0338", gtd->input_len - gtd->input_cur + 3, gtd->ses->cols - 16 - gtd->input_len, node->left);
+		if (gtd->input_len == gtd->input_cur)
+		{
+			input_printf("\033[0K]   %.*s\033[%dG", gtd->ses->cols - 16 - gtd->input_len, node->left, gtd->input_pos + 11);
+		}
+		else
+		{
+			input_printf("\033[%dC\033[0K]\033[3C%.*s\033%dG", gtd->input_len - gtd->input_cur, gtd->ses->cols - 16 - gtd->input_len, node->left, gtd->input_pos + 1);
+		}
 	}
 	else
 	{
-		input_printf("\033[%dC\033[0K\033[%dD", gtd->input_len - gtd->input_cur + 3, gtd->input_len - gtd->input_cur + 3);
+		if (gtd->input_len == gtd->input_cur)
+		{
+			input_printf("\033[0K]\033[%dG", gtd->input_pos + 11);
+		}
+		else
+		{
+			input_printf("\033[%dC\033[0K]\033[%dG", gtd->input_len - gtd->input_cur, gtd->input_pos + 11);
+		}
 	}
 	pop_call();
 }
@@ -527,7 +698,7 @@ DO_CURSOR(cursor_insert)
 	}
 	else
 	{
-		tintin_printf(NULL, "#SYNTAX: #CURSOR {INSERT} {ON|OFF}");
+		tintin_printf(NULL, "#SYNTAX: #CURSOR {INSERT} {ON|OFF}.");
 	}
 }
 
@@ -544,6 +715,34 @@ DO_CURSOR(cursor_left)
 	}
 }
 
+DO_CURSOR(cursor_left_word)
+{
+	int index_cur;
+
+	if (gtd->input_cur == 0)
+	{
+		return;
+	}
+
+	index_cur = gtd->input_cur;
+
+	while (gtd->input_cur && gtd->input_buf[gtd->input_cur - 1] == ' ')
+	{
+		gtd->input_cur--;
+		gtd->input_pos--;
+	}
+
+	while (gtd->input_cur && gtd->input_buf[gtd->input_cur - 1] != ' ')
+	{
+		gtd->input_cur--;
+		gtd->input_pos--;
+	}
+
+	input_printf("\033[%dD", index_cur - gtd->input_cur);
+
+	cursor_check_line("");
+}
+
 
 DO_CURSOR(cursor_paste_buffer)
 {
@@ -575,14 +774,7 @@ DO_CURSOR(cursor_redraw_input)
 
 DO_CURSOR(cursor_redraw_line)
 {
-	if (gtd->input_pos)
-	{
-		input_printf("\033[%dD\033[0K\0337", gtd->input_pos);
-	}
-	else
-	{
-		input_printf("\033[0K\0337");
-	}
+	input_printf("\033[%dG\033[%dX", gtd->input_off, gtd->input_len);
 
 	if (gtd->input_pos != gtd->input_cur)
 	{
@@ -610,32 +802,28 @@ DO_CURSOR(cursor_redraw_line)
 			}
 		}
 	}
-	if (gtd->input_cur != gtd->input_pos)
+
+	if (gtd->input_pos != gtd->input_cur)
 	{
-		input_printf("<%.*s\0338", gtd->ses->cols - 1, &gtd->input_buf[gtd->input_cur - gtd->input_pos + 1]);
+		input_printf("<%.*s\033[%dG", gtd->ses->cols - 1, &gtd->input_buf[gtd->input_cur - gtd->input_pos + 1], gtd->input_pos+gtd->input_off);
 	}
 	else
 	{
-		input_printf("%.*s\0338", gtd->ses->cols, &gtd->input_buf[gtd->input_cur - gtd->input_pos]);
+		input_printf("%.*s\033[%dG", gtd->ses->cols, &gtd->input_buf[gtd->input_cur - gtd->input_pos], gtd->input_pos+gtd->input_off);
 	}
 
 	cursor_fix_line("");
-
-	if (gtd->input_pos)
-	{
-		input_printf("\033[%dC", gtd->input_pos);
-	}
 }
 
 DO_CURSOR(cursor_fix_line)
 {
 	if (gtd->input_len - gtd->input_cur + gtd->input_pos > gtd->ses->cols)
 	{
-		input_printf("\0337\033[%dG%.1s>\0338", gtd->ses->cols - 1, &gtd->input_buf[gtd->input_cur + gtd->ses->cols - gtd->input_pos-2]);
+		input_printf("\033[%dG%.1s>\033[%dG", gtd->ses->cols - 1, &gtd->input_buf[gtd->input_cur + gtd->ses->cols - gtd->input_pos-2], gtd->input_pos+gtd->input_off);
 	}
 	else if (gtd->input_len - gtd->input_cur + gtd->input_pos == gtd->ses->cols)
 	{
-		input_printf("\0337\033[%dG%.2s\0338",  gtd->ses->cols - 1, &gtd->input_buf[gtd->input_cur + gtd->ses->cols - gtd->input_pos-2]);
+		input_printf("\033[%dG%.2s\033%dG",  gtd->ses->cols - 1, &gtd->input_buf[gtd->input_cur + gtd->ses->cols - gtd->input_pos-2], gtd->input_pos+gtd->input_off);
 	}
 }
 
@@ -651,7 +839,33 @@ DO_CURSOR(cursor_right)
 	cursor_check_line("");
 }
 
+DO_CURSOR(cursor_right_word)
+{
+	int index_cur;
 
+	if (gtd->input_cur == gtd->input_len)
+	{
+		return;
+	}
+
+	index_cur = gtd->input_cur;
+
+	while (gtd->input_cur < gtd->input_len && gtd->input_buf[gtd->input_cur] == ' ')
+	{
+		gtd->input_cur++;
+		gtd->input_pos++;
+	}
+
+	while (gtd->input_cur < gtd->input_len && gtd->input_buf[gtd->input_cur] != ' ')
+	{
+		gtd->input_cur++;
+		gtd->input_pos++;
+	}
+
+	input_printf("\033[%dC", gtd->input_cur - index_cur);
+
+	cursor_check_line("");
+}
 
 DO_CURSOR(cursor_suspend)
 {
@@ -687,6 +901,13 @@ DO_CURSOR(cursor_tab)
 		input_now++;
 	}
 
+	if (gtd->ses->list[LIST_TAB]->f_node == NULL)
+	{
+		cursor_auto_tab(arg);
+
+		return;
+	}
+
 	for (node = gtd->ses->list[LIST_TAB]->f_node ; node ; node = node->next)
 	{
 		substitute(gtd->ses, node->left, tab, SUB_VAR|SUB_FUN);
@@ -711,3 +932,96 @@ DO_CURSOR(cursor_tab)
 		}
 	}
 }
+
+DO_CURSOR(cursor_auto_tab)
+{
+	char tab[BUFFER_SIZE], buf[BUFFER_SIZE];
+	int input_now, scroll_cnt, line_cnt;
+
+	if (gtd->input_len == 0)
+	{
+		return;
+	}
+
+	input_now = gtd->input_len - 1;
+
+	while (input_now && gtd->input_buf[input_now] != ' ')
+	{
+		input_now--;
+	}
+
+	if (input_now != 0 && input_now == gtd->input_len - 1)
+	{
+		return;
+	}
+
+	if (input_now)
+	{
+		input_now++;
+	}
+
+	line_cnt = 0;
+
+	scroll_cnt = gtd->ses->scroll_row;
+
+	do
+	{
+		if (scroll_cnt == gtd->ses->scroll_max -1)
+		{
+			scroll_cnt = 0;
+		}
+		else
+		{
+			scroll_cnt++;
+		}
+
+		if (gtd->ses->buffer[scroll_cnt] == NULL)
+		{
+			break;
+		}
+
+		if (str_hash_grep(gtd->ses->buffer[scroll_cnt], FALSE))
+		{
+			continue;
+		}
+
+		if (line_cnt++ >= gtd->ses->auto_tab)
+		{
+			break;
+		}
+
+		strip_vt102_codes(gtd->ses->buffer[scroll_cnt], buf);
+
+		arg = buf;
+
+		while (*arg)
+		{
+			arg = get_arg_stop_spaces(arg, tab);
+
+			if (!strncmp(tab, &gtd->input_buf[input_now], strlen(&gtd->input_buf[input_now])))
+			{
+				if (gtd->input_cur == gtd->input_len)
+				{
+					input_printf("\033[%dD\033[%dP%s", gtd->input_len - input_now, gtd->input_len - input_now, tab);
+				}
+				else
+				{
+					input_printf("\033[%dC\033[%dD\033[%dP%s", gtd->input_len - gtd->input_cur, gtd->input_len - input_now, gtd->input_len - input_now, tab);
+				}
+				strcpy(&gtd->input_buf[input_now], tab);
+
+				gtd->input_len = input_now + strlen(tab);
+				gtd->input_cur = gtd->input_len;
+				gtd->input_pos = gtd->input_len;
+
+				return;
+			}
+
+			if (*arg == ';')
+			{
+				arg++;
+			}
+		}
+	}
+	while (scroll_cnt != gtd->ses->scroll_row);
+}
