@@ -70,14 +70,14 @@ DO_COMMAND(do_highlight)
 	{
 		if (get_highlight_codes(ses, right, buf) == FALSE)
 		{
-			tintin_printf2(ses, "#HIGHLIGHT: VALID COLORS ARE:\r\n");
+			tintin_printf2(ses, "#HIGHLIGHT: VALID COLORS ARE:\n");
 			tintin_printf2(ses, "reset, bold, light, faint, dim, dark, underscore, blink, reverse, black, red, green, yellow, blue, magenta, cyan, white, b black, b red, b green, b yellow, b blue, b magenta, b cyan, b white");
 		}
 		else
 		{
 			updatenode_list(ses, left, right, rank, LIST_HIGHLIGHT);
 
-			show_message(ses, LIST_HIGHLIGHT, "#OK. {%s} NOW HIGHLIGHTS {%s} @ {%s}", left, right, rank);
+			show_message(ses, LIST_HIGHLIGHT, "#OK. {%s} NOW HIGHLIGHTS {%s} @ {%s}.", left, right, rank);
 		}
 	}
 	return ses;
@@ -91,68 +91,30 @@ DO_COMMAND(do_unhighlight)
 	return ses;
 }
 
-/*
-void check_all_highlights(struct session *ses, char **original, char *line)
-{
-	struct listnode *node;
-	char *pt1, *pt2, *pt3;
-	char *buf1, buf2[BUFFER_SIZE], buf3[BUFFER_SIZE], temp[BUFFER_SIZE];
-
-	for (node = ses->list[LIST_HIGHLIGHT]->f_node ; node ; node = node->next)
-	{
-		if (check_one_action(line, *original, node->left, ses))
-		{
-			substitute(ses, node->left, &buf1, SUB_VAR|SUB_FUN|SUB_ARG|SUB_ANC|SUB_ESC);
-
-			pt1 = strstr(*original, buf1) ? *original : line;
-			pt2 = strstr(pt1, buf1);
-
-			if (pt2 == NULL)
-			{
-				continue;
-			}
-			*pt2 = 0;
-			pt3 = pt2 + strlen(buf1);
-
-			strip_non_vt102_codes(pt1, buf2);
-			strip_vt102_codes_non_graph(buf2, buf3);
-
-			get_highlight_codes(ses, node->right, buf2);
-
-			sprintf(temp, "%s%s%s\033[0m%s%s", pt1, buf2, buf1, buf3, pt3);
-
-			substitute(ses, temp, original, 0);
-		}
-	}
-}
-*/
-
 void check_all_highlights(struct session *ses, char *original, char *line)
 {
 	struct listnode *node;
 	char *pt1, *pt2;
-	char match[BUFFER_SIZE], color[BUFFER_SIZE], reset[BUFFER_SIZE], output[BUFFER_SIZE], temp[BUFFER_SIZE];
+	char match[BUFFER_SIZE], color[BUFFER_SIZE], reset[BUFFER_SIZE], output[BUFFER_SIZE], plain[BUFFER_SIZE];
 
 	for (node = ses->list[LIST_HIGHLIGHT]->f_node ; node ; node = node->next)
 	{
-		if (check_one_action(line, original, node->left, ses))
+		if (check_one_regexp(ses, node, line, original, 0))
 		{
-			substitute(ses, node->left, match, SUB_VAR|SUB_FUN|SUB_ARG|SUB_ANC|SUB_ESC);
-
-			if (strstr(original, match) == NULL)
+			if (*gtd->vars[0] == 0)
 			{
 				continue;
 			}
 
+			strcpy(match, gtd->vars[0]);
+
 			*output = *reset = 0;
 
 			pt1 = original;
 
 			get_highlight_codes(ses, node->right, color);
 
-			strip_vt102_codes(match, temp);
-
-			strcat(color, temp);
+			strip_vt102_codes(match, plain);
 
 			while ((pt2 = strstr(pt1, match)) != NULL)
 			{
@@ -160,13 +122,13 @@ void check_all_highlights(struct session *ses, char *original, char *line)
 
 				get_color_codes(reset, pt1, reset);
 
-				cat_sprintf(output, "%s%s\033[0m%s", pt1, color, reset);
+				cat_sprintf(output, "%s%s%s\033[0m%s", pt1, color, plain, reset);
 
 				pt1 = pt2 + strlen(match);
 			}
 			strcat(output, pt1);
 
-			substitute(ses, output, original, 0);
+			strcpy(original, output);
 		}
 	}
 }
