@@ -31,31 +31,39 @@ DO_COMMAND(do_history)
 {
 	char left[BUFFER_SIZE], right[BUFFER_SIZE];
 	int cnt;
-	struct listroot *root;
-	struct listnode *node;
 
-	root = gts->list[LIST_HISTORY];
+	arg = get_arg_in_braces(arg, left,  FALSE);
 
-	arg = get_arg_in_braces(arg, left,  0);
-	arg = get_arg_in_braces(arg, right, 1);
+	arg = get_arg_in_braces(arg, right, TRUE);
+	substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
-	switch (left[0])
+	if (*left == 0)
 	{
-		case 'd':
-			deletenode_list(gts, root->l_node, LIST_HISTORY);
-			break;
+		tintin_header(ses, " HISTORY COMMANDS ");
 
-		case 'i':
-			insertnode_list(gts, right, "", "", LIST_HISTORY);
-			break;
+		for (cnt = 0 ; *history_table[cnt].name != 0 ; cnt++)
+		{
+			tintin_printf2(ses, "  [%-13s] %s", history_table[cnt].name, history_table[cnt].desc);
+		}
+		tintin_header(ses, "");
 
-		default:
-			for (cnt = 0, node = root->f_node ; node ; node = node->next)
-			{
-				tintin_printf2(ses, "%5d - %s", cnt++, node->left);
-			}
-			break;
+		return ses;
 	}
+
+	for (cnt = 0 ; *history_table[cnt].name ; cnt++)
+	{
+		if (!is_abbrev(left, history_table[cnt].name))
+		{
+			continue;
+		}
+
+		history_table[cnt].fun(ses, right);
+
+		return ses;
+	}
+
+	do_history(ses, "");
+
 	return ses;
 }
 
@@ -113,7 +121,6 @@ void add_line_history(struct session *ses, char *line)
 	return;
 }
 
-
 void search_line_history(struct session *ses, char *line)
 {
 	struct listroot *root;
@@ -133,38 +140,100 @@ void search_line_history(struct session *ses, char *line)
 	tintin_printf2(ses, "#REPEAT: NO MATCH FOUND FOR '%s'", line);
 }
 
-int write_history(struct session *ses, char *filename)
+DO_HISTORY(history_character)
 {
-	struct listnode *node;
-	FILE *file;
+	gtd->repeat_char = *arg;
 
-	file = fopen(filename, "w");
-
-	if (file == NULL)
-	{
-		return FALSE;
-	}
-
-	for (node = ses->list[LIST_HISTORY]->f_node ; node ; node = node->next)
-	{
-		fprintf(file, "%s\n", node->left);
-	}
-
-	fclose(file);
-
-	return TRUE;
+	show_message(ses, LIST_HISTORY, "#HISTORY CHARACTER SET TO {%c}.", gtd->repeat_char);
 }
 
-int read_history(struct session *ses, char *filename)
+DO_HISTORY(history_delete)
+{
+	if (gts->list[LIST_HISTORY]->l_node)
+	{
+		deletenode_list(gts, gts->list[LIST_HISTORY]->l_node, LIST_HISTORY);
+	}
+
+	return;
+}
+
+DO_HISTORY(history_insert)
+{
+	struct listroot *root;
+	struct listnode *node;
+
+	root = ses->list[LIST_HISTORY];
+
+	if (root->l_node && !strcmp(arg, root->l_node->left))
+	{
+		return;
+	}
+
+	for (node = root->f_node ; node ; node = node->next)
+	{
+		if (!strcmp(arg, node->left))
+		{
+			if (node == gtd->input_his)
+			{
+				gtd->input_his = NULL;
+			}
+			deletenode_list(ses, node, LIST_HISTORY);
+			break;
+		}
+	}
+
+	insertnode_list(ses, arg, "", "", LIST_HISTORY);
+
+	while (root->count > gtd->history_size)
+	{
+		if (root->f_node == gtd->input_his)
+		{
+			gtd->input_his = NULL;
+		}
+		deletenode_list(ses, root->f_node, LIST_HISTORY);
+	}
+
+	return;
+}
+
+DO_HISTORY(history_list)
+{
+	struct listroot *root;
+	struct listnode *node;
+	int cnt = 1;
+
+	root = ses->list[LIST_HISTORY];
+
+	for (node = root->f_node; node ; node = node->next)
+	{
+		tintin_printf2(ses, "%6d - %s", cnt++, node->left);
+	}
+	return;
+}
+
+DO_HISTORY(history_read)
 {
 	FILE *file;
+	struct listroot *root;
 	char *cptr, buffer[BUFFER_SIZE];
 
-	file = fopen(filename, "r");
+	root = ses->list[LIST_HISTORY];
+
+	file = fopen(arg, "r");
 
 	if (file == NULL)
 	{
-		return FALSE;
+		tintin_printf2(ses, "#HISTORY: COULDN'T OPEN FILE {%s} TO READ.", arg);
+		return;
+	}
+
+	while (root->f_node)
+	{
+		if (root->f_node == gtd->input_his)
+		{
+			gtd->input_his = NULL;
+		}
+		deletenode_list(ses, root->f_node, LIST_HISTORY);
 	}
 
 	while (fgets(buffer, BUFFER_SIZE-1, file))
@@ -185,5 +254,42 @@ int read_history(struct session *ses, char *filename)
 
 	fclose(file);
 
-	return TRUE;
+	return;
+}
+
+DO_HISTORY(history_size)
+{
+	if (atoi(arg) < 1 || atoi(arg) > 100000)
+	{
+		tintin_printf(ses, "#HISTORY SIZE: PROVIDE A NUMBER BETWEEN 1 and 100,000");
+	}
+	else
+	{
+		gtd->history_size = atoi(arg);
+	}
+	return;
+}
+
+DO_HISTORY(history_write)
+{
+	struct listnode *node;
+	FILE *file;
+
+	file = fopen(arg, "w");
+
+	if (file == NULL)
+	{
+		tintin_printf2(ses, "#HISTORY: COULDN'T OPEN FILE {%s} TO WRITE.", arg);
+
+		return;
+	}
+
+	for (node = ses->list[LIST_HISTORY]->f_node ; node ; node = node->next)
+	{
+		fprintf(file, "%s\n", node->left);
+	}
+
+	fclose(file);
+
+	return;
 }
