@@ -29,40 +29,39 @@
 
 void process_input(void)
 {
-	char line[STRING_SIZE];
+	int flags;
 
-	line[0] = 127;
-
-	if (HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_SGA)
-	&& !HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO))
+	if (HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_SGA)	&& !HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO))
 	{
-		read_key(line);
+		read_key();
 	}
 	else
 	{
-		read_line(line);
+		read_line();
 	}
 
-	if (*line == 127)
+	if (!HAS_BIT(gtd->flags, TINTIN_FLAG_PROCESSINPUT))
 	{
 		return;
 	}
 
+	DEL_BIT(gtd->flags, TINTIN_FLAG_PROCESSINPUT);
+
 	if (gtd->chat && gtd->chat->paste_time)
 	{
-		chat_paste(line, NULL);
+		chat_paste(gtd->input_buf, NULL);
 
 		return;
 	}
 
 	if (HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO))
 	{
-		add_line_history(gts, line);
+		add_line_history(gts, gtd->input_buf);
 	}
 
 	if (HAS_BIT(gtd->ses->telopts, TELOPT_FLAG_ECHO))
 	{
-		echo_command(gtd->ses, line, FALSE);
+		echo_command(gtd->ses, gtd->input_buf, FALSE);
 	}
 	else
 	{
@@ -71,28 +70,36 @@ void process_input(void)
 
 	if (gtd->ses->scroll_line != -1)
 	{
-		buffer_e("");
+		buffer_end(gtd->ses, "");
 	}
 
-	SET_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND);
+	flags = TINTIN_FLAG_USERCOMMAND | (HAS_BIT(gtd->ses->flags, SES_FLAG_VERBATIM) ? TINTIN_FLAG_VERBATIM : 0);
 
-	pre_parse_input(gtd->ses, line, SUB_NONE);
+	SET_BIT(gtd->flags, flags);
 
-	DEL_BIT(gtd->flags, TINTIN_FLAG_USERCOMMAND);
+	gtd->ses = script_driver(gtd->ses, -1, gtd->input_buf);
+
+	DEL_BIT(gtd->flags, flags);
+
+	check_all_events(gtd->ses, 0, 1, "RECEIVED INPUT", gtd->input_buf);
 
 	if (IS_SPLIT(gtd->ses))
 	{
 		erase_toeol();
 	}
+
+	gtd->input_buf[0] = 0;
 }
 
-void read_line(char *line)
+void read_line()
 {
 	char buffer[STRING_SIZE];
 	struct listnode *node;
 	struct listroot *root;
 	int len, cnt, match;
 
+	gtd->input_buf[gtd->input_len] = 0;
+
 	len = read(0, buffer, 1);
 
 	buffer[len] = 0;
@@ -117,7 +124,7 @@ void read_line(char *line)
 
 			if (!strcmp(gtd->macro_buf, node->pr))
 			{
-				pre_parse_input(gtd->ses, node->right, SUB_NONE);
+				script_driver(gtd->ses, LIST_MACRO, node->right);
 
 				gtd->macro_buf[0] = 0;
 
@@ -129,7 +136,7 @@ void read_line(char *line)
 			}
 		}
 
-		for (cnt = 0 ; *cursor_table[cnt].code != 0 ; cnt++)
+		for (cnt = 0 ; *cursor_table[cnt].fun != NULL ; cnt++)
 		{
 			if (!strcmp(gtd->macro_buf, cursor_table[cnt].code))
 			{
@@ -162,29 +169,7 @@ void read_line(char *line)
 		switch (gtd->macro_buf[cnt])
 		{
 			case 10:
-				input_printf("\n");
-
-				gtd->input_buf[gtd->input_len] = 0;
-
-				strcpy(line, gtd->input_buf);
-
-				gtd->input_len    = 0;
-				gtd->input_cur    = 0;
-				gtd->input_pos    = 0;
-				gtd->input_buf[0] = 0;
-				gtd->macro_buf[0] = 0;
-				gtd->input_tmp[0] = 0;
-
-				if (HAS_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH))
-				{
-					DEL_BIT(gtd->flags, TINTIN_FLAG_HISTORYSEARCH);
-
-					if (gtd->input_his)
-					{
-						strcpy(line, gtd->input_his->left);
-					}
-				}
-				gtd->input_his = NULL;
+				cursor_enter("");
 				break;
 
 			default:
@@ -230,7 +215,7 @@ void read_line(char *line)
 	}
 }
 
-void read_key(char *line)
+void read_key(void)
 {
 	char buffer[BUFFER_SIZE];
 	struct listnode *node;
@@ -239,7 +224,7 @@ void read_key(char *line)
 
 	if (gtd->input_buf[0] == gtd->tintin_char)
 	{
-		read_line(line);
+		read_line();
 
 		return;
 	}
@@ -269,7 +254,7 @@ void read_key(char *line)
 
 			if (!strcmp(gtd->macro_buf, node->pr))
 			{
-				pre_parse_input(gtd->ses, buffer, SUB_NONE);
+				script_driver(gtd->ses, LIST_MACRO, node->right);
 
 				gtd->macro_buf[0] = 0;
 				return;
@@ -291,10 +276,22 @@ void read_key(char *line)
 		switch (gtd->macro_buf[cnt])
 		{
 			case '\r':
+				printf("\n\[1;31mfound \\r!\n");
+				break;
+
 			case '\n':
 				gtd->input_buf[0] = 0;
 				gtd->macro_buf[0] = 0;
-				socket_printf(gtd->ses, 1, "%c", '\r'); 
+				gtd->input_len = 0;
+
+				if (HAS_BIT(gtd->ses->flags, SES_FLAG_RUN))
+				{
+					socket_printf(gtd->ses, 1, "%c", '\r');
+				}
+				else
+				{
+					socket_printf(gtd->ses, 2, "%c%c", '\r', '\n');
+				}
 				break;
 
 			default:
@@ -347,20 +344,25 @@ void convert_meta(char *input, char *output)
 				break;
 
 			case 127:
-				*pto++ = *pti++;
+				*pto++ = '\\';
+				*pto++ = 'b';
+				pti++;
 				break;
 
-			case '\n':
 			case '\r':
+				*pto++ = '\\';
+				*pto++ = 'r';
+				pti++;
+				break;
+			case '\n':
 				*pto++ = *pti++;
 				break;
 
 			default:
-				if (*pti > 0 && *pti < 30)
+				if (*pti < 32)
 				{
 					*pto++ = '\\';
-					*pto++ = 'C';
-					*pto++ = '-';
+					*pto++ = 'c';
 					*pto++ = 'a' + *pti - 1;
 					pti++;
 					break;
@@ -389,13 +391,8 @@ void unconvert_meta(char *input, char *output)
 			case '\\':
 				switch (pti[1])
 				{
-					case 'e':
-						*pto++  = ESCAPE;
-						pti    += 2;
-						break;
-
 					case 'C':
-						if (pti[2] == '-')
+						if (pti[2] == '-' && pti[3])
 						{
 							*pto++  = pti[3] - 'a' + 1;
 							pti    += 4;
@@ -406,6 +403,37 @@ void unconvert_meta(char *input, char *output)
 						}
 						break;
 
+					case 'c':
+						*pto++ = pti[2] % 32;
+						pti += 3;
+						break;
+
+					case 'b':
+						*pto++  = 127;
+						pti    += 2;
+						break;
+
+					case 'e':
+						*pto++  = ESCAPE;
+						pti    += 2;
+						break;
+
+					case 't':
+						*pto++  = '\t';
+						pti    += 2;
+						break;
+
+					case 'x':
+						if (pti[2] && pti[3])
+						{
+							*pto++ = hex_number(&pti[2]);
+							pti += 4;
+						}
+						else
+						{
+							*pto++ = *pti++;
+						}
+						break;
 					default:
 						*pto++ = *pti++;
 						break;
@@ -420,13 +448,17 @@ void unconvert_meta(char *input, char *output)
 	*pto = 0;
 }
 
+/*
+	Currenly only used in split mode.
+*/
+
 void echo_command(struct session *ses, char *line, int newline)
 {
 	char buffer[STRING_SIZE], result[STRING_SIZE];
 
 	if (HAS_BIT(ses->flags, SES_FLAG_SPLIT))
 	{
-		sprintf(buffer, "\033[0;37m%s", line);
+		sprintf(buffer, "%s%s\033[0m", ses->cmd_color, line);
 	}
 	else
 	{
@@ -451,7 +483,7 @@ void echo_command(struct session *ses, char *line, int newline)
 			}
 			else
 			{
-				tintin_printf2(ses, "%s%s", buffer);
+				tintin_printf2(ses, "%s", buffer);
 			}
 
 			DEL_BIT(ses->flags, SES_FLAG_SCROLLSTOP);
@@ -471,14 +503,7 @@ void echo_command(struct session *ses, char *line, int newline)
 		}
 		else
 		{
-			if (ses->more_output[0])
-			{
-				sprintf(result, "%s\033[0;37m%s", ses->more_output, line);
-			}
-			else
-			{
-				sprintf(result, "\033[0;37m%s", line);
-			}
+			sprintf(result, "%s%s", ses->more_output, buffer);
 		}
 		add_line_buffer(ses, buffer, -1);
 
