@@ -37,11 +37,13 @@ DO_COMMAND(do_all)
 	{
 		get_arg_in_braces(arg, left, TRUE);
 
+		substitute(ses, left, left, SUB_VAR|SUB_FUN);
+
 		for (sesptr = gts->next ; sesptr ; sesptr = next_ses)
 		{
 			next_ses = sesptr->next;
 
-			pre_parse_input(sesptr, left, SUB_NONE);
+			script_driver(sesptr, -1, left);
 		}
 	}
 	else
@@ -73,7 +75,7 @@ DO_COMMAND(do_commands)
 		{
 			continue;
 		}
-		if (strlen(buf) + 20 > ses->cols)
+		if ((int) strlen(buf) + 20 > ses->cols)
 		{
 			tintin_puts2(ses, buf);
 			buf[0] = 0;
@@ -101,7 +103,8 @@ DO_COMMAND(do_cr)
 
 DO_COMMAND(do_echo)
 {
-	char temp[BUFFER_SIZE];
+	char temp[BUFFER_SIZE], output[BUFFER_SIZE];
+	int lnf;
 
 	sprintf(temp, "result %s", arg);
 
@@ -109,17 +112,44 @@ DO_COMMAND(do_echo)
 
 	do_format(ses, temp);
 
-	substitute(ses, "$result", temp, SUB_VAR);
+	substitute(ses, "$result", temp, SUB_VAR|SUB_FUN|SUB_COL);
 
-	if (*temp)
+	lnf = !str_suffix(temp, "\\");
+
+	substitute(ses, temp, temp, SUB_ESC);
+
+	if (strip_vt102_strlen(ses->more_output) != 0)
 	{
-		do_showme(ses, temp);
+		sprintf(output, "\n\033[0m%s\033[0m", temp);
+	}
+	else
+	{
+		sprintf(output, "\033[0m%s\033[0m", temp);
+	}
+
+	add_line_buffer(ses, output, lnf);
+
+	if (ses != gtd->ses)
+	{
+		return ses;
+	}
+
+	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
+	{
+		save_pos(ses);
+		goto_rowcol(ses, ses->bot_row, 1);
+	}
+
+	printline(ses, output, lnf);
+
+	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
+	{
+		restore_pos(ses);
 	}
 
 	return ses;
 }
 
-
 DO_COMMAND(do_end)
 {
 	if (*arg)
@@ -155,19 +185,20 @@ DO_COMMAND(do_forall)
 		{
 			arg = get_arg_in_braces(arg, temp, FALSE);
 
-			internal_variable(ses, "{forall} {%s}", temp);
-
 			RESTRING(gtd->cmds[0], temp);
 
-			ses = pre_parse_input(ses, right, SUB_CMD);
+			substitute(ses, right, temp, SUB_CMD);
+
+			ses = script_driver(ses, -1, temp);
 		}
 	}
 	return ses;
 }
 
-
 DO_COMMAND(do_gagline)
 {
+	tintin_printf2(ses, "#GAGLINE: Old command, please use #LINE GAG instead.");
+
 	SET_BIT(ses->flags, SES_FLAG_GAG);
 
 	return ses;
@@ -178,8 +209,6 @@ DO_COMMAND(do_info)
 {
 	int cnt;
 
-	printf("--%s--\n", arg);
-
 	if (*arg == 'c')
 	{
 		show_cpu(ses);
@@ -187,6 +216,13 @@ DO_COMMAND(do_info)
 		return ses;
 	}
 
+	if (*arg == 's')
+	{
+		dump_stack();
+
+		return ses;
+	}
+
 	tintin_header(ses, " INFORMATION ");
 
 	for (cnt = 0 ; cnt < LIST_MAX ; cnt++)
@@ -221,49 +257,41 @@ DO_COMMAND(do_loop)
 	arg = get_arg_in_braces(arg, right, 0);
 
 
-	if (*command == 0 || (*left == 0 && *right == 0))
+	if (*command == 0 || *right == 0)
 	{
 		show_message(ses, LIST_MATH, "#LOOP: INVALID ARGUMENTS.");
 
 		return ses;
 	}
 
-	if (*left && *right)
+	bound1 = (long long) get_number(ses, left);
+	bound2 = (long long) get_number(ses, right);
+
+	if (bound1 <= bound2)
 	{
-		bound1 = (long long) get_number(ses, left);
-		bound2 = (long long) get_number(ses, right);
-
-		if (bound1 <= bound2)
-		{
-			step = 1;
-			bound2++;
-		}
-		else
-		{
-			step = -1;
-			bound2--;
-		}
-
-		for (counter = bound1 ; counter != bound2 ; counter += step)
-		{
-			gtd->cmds[0] = refstring(gtd->cmds[0], "%lld", counter);
-
-			internal_variable(ses, "{loop} {%lld}", counter);
-
-			ses = pre_parse_input(ses, command, SUB_CMD);
-		}
+		step = 1;
+		bound2++;
 	}
 	else
 	{
-		while ((long long) get_number(ses, left))
-		{
-			ses = pre_parse_input(ses, command, SUB_NONE);
-		}
+		step = -1;
+		bound2--;
 	}
+
+	for (counter = bound1 ; counter != bound2 ; counter += step)
+	{
+		gtd->cmds[0] = refstring(gtd->cmds[0], "%lld", counter);
+
+		substitute(ses, command, temp, SUB_CMD);
+
+		ses = script_driver(ses, -1, temp);
+	}
+
 	return ses;
 }
 
 
+
 DO_COMMAND(do_nop)
 {
 	return ses;
@@ -291,7 +319,9 @@ DO_COMMAND(do_parse)
 #ifdef BIG5
 			if (left[cnt] & 0x80 && left[cnt+1] != 0)
 			{
-				sprintf(temp, "%c%c", left[cnt], left[++cnt]));
+				sprintf(temp, "%c%c", left[cnt], left[cnt+1]);
+
+				cnt++;
 			}
 			else
 			{
@@ -303,9 +333,9 @@ DO_COMMAND(do_parse)
 
 			RESTRING(gtd->cmds[0], temp);
 
-			internal_variable(ses, "{parse} {%s}", gtd->cmds[0]);
+			substitute(ses, right, temp, SUB_CMD);
 
-			ses = pre_parse_input(ses, right, SUB_CMD);
+			ses = script_driver(ses, -1, temp);
 		}
 	}
 	return ses;
@@ -314,8 +344,6 @@ DO_COMMAND(do_parse)
 
 DO_COMMAND(do_return)
 {
-	SET_BIT(ses->flags, SES_FLAG_BREAK);
-
 	if (*arg)
 	{
 		internal_variable(ses, "{result} %s", arg);
@@ -327,29 +355,36 @@ DO_COMMAND(do_send)
 {
 	char left[BUFFER_SIZE];
 
+	push_call("do_send(%p,%p)",ses,arg);
+
 	get_arg_in_braces(arg, left, TRUE);
 
 	write_mud(ses, left, SUB_VAR|SUB_FUN|SUB_ESC|SUB_EOL);
 
+	pop_call();
 	return ses;
 }
 
 DO_COMMAND(do_showme)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], output[STRING_SIZE];
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], temp[STRING_SIZE];
+	int lnf;
 
-	arg = get_arg_in_braces(arg, left,  1);
-	arg = get_arg_in_braces(arg, right, 0);
+	arg = get_arg_in_braces(arg, left, TRUE);
+	lnf = !str_suffix(left, "\\");
+	substitute(ses, left, temp, SUB_VAR|SUB_FUN|SUB_COL|SUB_ESC);
 
-	substitute(ses, left, left, SUB_COL|SUB_ESC);
+	arg = get_arg_in_braces(arg, right, FALSE);
+	substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
 	if (*right)
 	{
-		do_one_prompt(ses, left, atoi(right));
+		do_one_prompt(ses, temp, atoi(right));
 
 		return ses;
 	}
-	do_one_line(left, ses);
+
+	do_one_line(temp, ses);
 
 	if (HAS_BIT(ses->flags, SES_FLAG_GAG))
 	{
@@ -360,14 +395,14 @@ DO_COMMAND(do_showme)
 
 	if (strip_vt102_strlen(ses->more_output) != 0)
 	{
-		sprintf(output, "\r\n\033[0m%s\033[0m", left);
+		sprintf(right, "\n\033[0m%s\033[0m", temp);
 	}
 	else
 	{
-		sprintf(output, "\033[0m%s\033[0m", left);
+		sprintf(right, "\033[0m%s\033[0m", temp);
 	}
 
-	add_line_buffer(ses, output, FALSE);
+	add_line_buffer(ses, right, lnf);
 
 	if (ses != gtd->ses)
 	{
@@ -380,7 +415,7 @@ DO_COMMAND(do_showme)
 		goto_rowcol(ses, ses->bot_row, 1);
 	}
 
-	printline(ses, output, FALSE);
+	printline(ses, right, lnf);
 
 	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
 	{
@@ -396,6 +431,7 @@ DO_COMMAND(do_snoop)
 	char left[BUFFER_SIZE];
 
 	get_arg_in_braces(arg, left, 1);
+	substitute(ses, left, left, SUB_VAR|SUB_FUN);
 
 	if (*left)
 	{
@@ -439,6 +475,27 @@ DO_COMMAND(do_suspend)
 }
 
 
+DO_COMMAND(do_while)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE];
+
+	arg = get_arg_in_braces(arg, left, FALSE);
+	arg = get_arg_in_braces(arg, right, TRUE);
+
+	if (*left == 0 || *right == 0)
+	{
+		show_message(ses, LIST_MATH, "#SYNTAX: WHILE {CONDITION} {COMMANDS}");
+
+		return ses;
+	}
+
+	while (get_number(ses, left))
+	{
+		ses = script_driver(ses, -1, right);
+	}
+	return ses;
+}
+
 DO_COMMAND(do_zap)
 {
 	tintin_puts(ses, "");
