@@ -43,11 +43,12 @@
 
 #ifdef HAVE_GETADDRINFO
 
-int connect_mud(char *host, char *port, struct session *ses)
+int connect_mud(struct session *ses, char *host, char *port)
 {
 	int sock, error;
 	struct addrinfo *address;
 	static struct addrinfo hints;
+	char ip[100];
 
 	if (!is_number(port))
 	{
@@ -92,12 +93,15 @@ int connect_mud(char *host, char *port, struct session *ses)
 		return 0;
 	}
 
-	inet_ntop(address->ai_family, address->ai_addr, host, address->ai_addrlen);
-
 	if (fcntl(sock, F_SETFL, O_NDELAY|O_NONBLOCK) == -1)
 	{
 		perror("connect_mud: fcntl O_NDELAY|O_NONBLOCK");
 	}
+
+	getnameinfo(address->ai_addr, address->ai_addrlen, ip, 100, NULL, 0, NI_NUMERICHOST);
+
+	RESTRING(ses->ip, ip);
+
 	freeaddrinfo(address);
 
 	return sock;
@@ -105,7 +109,7 @@ int connect_mud(char *host, char *port, struct session *ses)
 
 #else
 
-int connect_mud(char *host, char *port, struct session *ses)
+int connect_mud(struct session *ses, char *host, char *port)
 {
 	int sock, d;
 	struct sockaddr_in sockaddr;
@@ -157,12 +161,14 @@ int connect_mud(char *host, char *port, struct session *ses)
 		perror("connect_mud: fcntl O_NDELAY|O_NONBLOCK");
 	}
 
+	RESTRING(ses->ip, inet_ntoa(sockaddr.sin_addr));
+
 	return sock;
 }
 
 #endif
 
-void write_line_mud(char *line, struct session *ses)
+void write_line_mud(struct session *ses, char *line, int size)
 {
 	push_call("write_line_mud(%p,%p)",line,ses);
 
@@ -182,7 +188,7 @@ void write_line_mud(char *line, struct session *ses)
 		return;
 	}
 
-	if (write(ses->socket, line, strlen(line)) == -1)
+	if (write(ses->socket, line, size) == -1)
 	{
 		syserr("write in write_line_mud");
 	}
@@ -193,12 +199,12 @@ void write_line_mud(char *line, struct session *ses)
 
 int read_buffer_mud(struct session *ses)
 {
-	unsigned char buf[STRING_SIZE];
+	unsigned char buffer[BUFFER_SIZE];
 	int size;
 
 	push_call("read_buffer_mud(%p)",ses);
 
-	size = read(ses->socket, buf, STRING_SIZE - 1);
+	size = read(ses->socket, buffer, BUFFER_SIZE - 1);
 
 	if (size <= 0)
 	{
@@ -206,9 +212,7 @@ int read_buffer_mud(struct session *ses)
 		return FALSE;
 	}
 
-	buf[size] = 0;
-
-	translate_telopts(ses, buf, size);
+	ses->read_len = translate_telopts(ses, buffer, size);
 
 	pop_call();
 	return TRUE;
@@ -261,7 +265,7 @@ void readmud(struct session *ses)
 			}
 		}
 
-		if (*line && ses->more_output[0])
+		if (ses->more_output[0])
 		{
 			if (ses->check_output)
 			{
@@ -269,9 +273,11 @@ void readmud(struct session *ses)
 
 				ses->more_output[0] = 0;
 			}
-			else if (HAS_BIT(gtd->ses->flags, SES_FLAG_SPLIT))
+			else if (*line && HAS_BIT(ses->flags, SES_FLAG_SPLIT))
 			{
 				sprintf(linebuf, "%s%s", ses->more_output, line);
+
+				ses->more_output[0] = 0;
 			}
 			else
 			{
