@@ -35,10 +35,14 @@
 
 DO_COMMAND(do_session)
 {
-	char left[BUFFER_SIZE];
+	char temp[BUFFER_SIZE], left[BUFFER_SIZE];
 	struct session *sesptr;
 	int cnt;
 
+	substitute(ses, arg, temp, SUB_VAR|SUB_FUN);
+
+	arg = temp;
+
 	arg = get_arg_in_braces(arg, left,  FALSE);
 
 	if (*left == 0)
@@ -190,7 +194,7 @@ struct session *new_session(struct session *ses, char *name, char *address, int
 		}
 	}
 
-	for (newsession = gts->next ; newsession ; newsession = newsession->next)
+	for (newsession = gts ; newsession ; newsession = newsession->next)
 	{
 		if (!strcmp(newsession->name, name))
 		{
@@ -201,15 +205,22 @@ struct session *new_session(struct session *ses, char *name, char *address, int
 		}
 	}
 
-	newsession                = calloc(1, sizeof(struct session));
+	newsession                = (struct session *) calloc(1, sizeof(struct session));
 
 	newsession->name          = strdup(name);
 	newsession->host          = strdup(host);
+	newsession->ip            = strdup("");
 	newsession->port          = strdup(port);
 
-	newsession->class         = strdup(gts->class);
+	newsession->group         = strdup(gts->group);
 	newsession->flags         = gts->flags;
 	newsession->telopts       = gts->telopts;
+	newsession->auto_tab      = gts->auto_tab;
+
+	newsession->cmd_color     = strdup(gts->cmd_color);
+
+	newsession->read_max      = gts->read_max;
+	newsession->read_buf      = (unsigned char *) calloc(1, gts->read_max);
 
 	gtd->ses                  = newsession;
 
@@ -230,18 +241,21 @@ struct session *new_session(struct session *ses, char *name, char *address, int
 
 	dirty_screen(newsession);
 
+	tintin_printf2(ses, "#Trying to connect to %s port %s.", newsession->host, newsession->port);
+
 	if (desc == 0)
 	{
 		connect_session(newsession);
 	}
 	else
 	{
-		SET_BIT(newsession->flags, SES_FLAG_CONNECTED);
+		SET_BIT(newsession->flags, SES_FLAG_CONNECTED|SES_FLAG_RUN);
 
 		SET_BIT(newsession->telopts, TELOPT_FLAG_SGA);
 		DEL_BIT(newsession->telopts, TELOPT_FLAG_ECHO);
 
-		gtd->ses         = newsession;
+		gtd->ses = newsession;
+
 		gtd->ses->socket = desc;
 	}
 
@@ -253,13 +267,11 @@ void connect_session(struct session *ses)
 {
 	int sock;
 
-	tintin_printf2(ses, "\n\r#Trying to connect to %s port %s.\n\r", ses->host, ses->port);
-
 	ses->connect_retry = utime() + gts->connect_retry;
 
 	reconnect:
 
-	sock = connect_mud(ses->host, ses->port, ses);
+	sock = connect_mud(ses, ses->host, ses->port);
 
 	if (sock == -1)
 	{
@@ -277,14 +289,16 @@ void connect_session(struct session *ses)
 
 		SET_BIT(ses->flags, SES_FLAG_CONNECTED);
 
-		tintin_printf(ses, "#SESSION '%s' CONNECTED TO '%s' PORT '%s'\n\r", ses->name, ses->host, ses->port);
+		tintin_printf2(ses, "");
+
+		tintin_printf(ses, "#SESSION '%s' CONNECTED TO '%s' PORT '%s'", ses->name, ses->host, ses->port);
 
 		if (atoi(ses->port) == TELNET_PORT)
 		{
 			init_telnet_session(ses);
 		}
 
-		check_all_events(ses, "SESSION CONNECTED");
+		check_all_events(ses, 0, 4, "SESSION CONNECTED", ses->name, ses->host, ses->ip, ses->port);
 
 		return;
 	}
@@ -339,7 +353,7 @@ void cleanup_session(struct session *ses)
 		}
 	}
 
-	check_all_events(ses, "SESSION DISCONNECTED");
+	check_all_events(ses, 0, 4, "SESSION DISCONNECTED", ses->name, ses->host, ses->ip, ses->port);
 
 	tintin_printf(gtd->ses, "");
 
@@ -350,13 +364,6 @@ void cleanup_session(struct session *ses)
 		gtd->ses = newactive_session();
 	}
 
-	do_killall(ses, NULL);
-
-	if (ses->map)
-	{
-		delete_map(ses);
-	}
-
 	if (ses->logfile)
 	{
 		fclose(ses->logfile);
@@ -367,15 +374,33 @@ void cleanup_session(struct session *ses)
 		fclose(ses->logline);
 	}
 
+	LINK(ses, gtd->dispose_next, gtd->dispose_prev);
+
 	pop_call();
-	push_call("cleanup_session(%p) (free)", ses);
+	return;
+}
+
+void dispose_session(struct session *ses)
+{
+	push_call("dispose_session(%p)", ses);
+
+	UNLINK(ses, gtd->dispose_next, gtd->dispose_prev);
+
+	do_killall(ses, NULL);
+
+	if (ses->map)
+	{
+		delete_map(ses);
+	}
 
 	init_buffer(ses, 0);
 
 	free(ses->name);
 	free(ses->host);
+	free(ses->ip);
 	free(ses->port);
-	free(ses->class);
+	free(ses->group);
+	free(ses->read_buf);
 
 	free(ses);
 
