@@ -36,14 +36,14 @@ void init_terminal()
 {
 	struct termios io;
 
-	if (tcgetattr(0, &gtd->terminal))
+	if (tcgetattr(0, &gtd->old_terminal))
 	{
 		perror("tcgetattr");
 
 		exit(errno);
 	}
 
-	io = gtd->terminal;
+	io = gtd->old_terminal;
 
 	/*
 		Canonical mode off
@@ -51,8 +51,10 @@ void init_terminal()
 
 	DEL_BIT(io.c_lflag, ICANON);
 
-	io.c_cc[VMIN]  = 1;
-	io.c_cc[VTIME] = 0;
+	io.c_cc[VMIN]   = 1;
+	io.c_cc[VTIME]  = 0;
+	io.c_cc[VSTART] = 255;
+	io.c_cc[VSTOP]  = 255;
 
 	/*
 		Make the terminalal as raw as possible
@@ -61,11 +63,10 @@ void init_terminal()
 /*
 	DEL_BIT(io.c_iflag, IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
 	DEL_BIT(io.c_oflag, OPOST);
-
 	DEL_BIT(io.c_cflag, CSIZE|PARENB);
 */
-	DEL_BIT(io.c_lflag, ECHO|ECHONL|IEXTEN);
-	DEL_BIT(io.c_lflag, ECHO|ECHONL);
+
+	DEL_BIT(io.c_lflag, ECHO|ECHONL|IEXTEN|ISIG);
 
 	SET_BIT(io.c_cflag, CS8);
 
@@ -75,11 +76,18 @@ void init_terminal()
 
 		exit(errno);
 	}
+
+	if (tcgetattr(0, &gtd->new_terminal))
+	{
+		perror("tcgetattr");
+
+		exit(errno);
+	}
 }
 
 void reset_terminal(void)
 {
-	tcsetattr(0, TCSANOW, &gtd->terminal);
+	tcsetattr(0, TCSANOW, &gtd->old_terminal);
 }
 
 void echo_off(struct session *ses)
