@@ -28,7 +28,6 @@
 #include "tintin.h"
 
 
-
 void printline(struct session *ses, char *str, int prompt)
 {
 	char wrapped_str[STRING_SIZE];
@@ -47,9 +46,14 @@ void printline(struct session *ses, char *str, int prompt)
 		return;
 	}
 
-	word_wrap(ses, str, wrapped_str, TRUE);
-
-/*	strcpy(wrapped_str, str); */
+	if (HAS_BIT(ses->flags, SES_FLAG_WORDWRAP))
+	{
+		word_wrap(ses, str, wrapped_str, TRUE);
+	}
+	else
+	{
+		strcpy(wrapped_str, str);
+	}
 
 	if (prompt)
 	{
@@ -57,7 +61,7 @@ void printline(struct session *ses, char *str, int prompt)
 	}
 	else
 	{
-		printf("%s\r\n", wrapped_str);
+		printf("%s\n", wrapped_str);
 	}
 	pop_call();
 	return;
@@ -89,11 +93,10 @@ int word_wrap(struct session *ses, char *textin, char *textout, int scroll)
 					break;
 
 				default:
-					if (*pti < 27)
+					if (*pti < 32)
 					{
 						*pto++ = '\\';
-						*pto++ = 'C';
-						*pto++ = '-';
+						*pto++ = 'c';
 						*pto++ = 'a' + *pti - 1;
 					}
 					else
@@ -132,7 +135,6 @@ int word_wrap(struct session *ses, char *textin, char *textout, int scroll)
 
 		if (*pti == '\n')
 		{
-			*pto++ = '\r';
 			*pto++ = *pti++;
 			cnt = cnt + 1;
 			los = pto;
@@ -143,7 +145,7 @@ int word_wrap(struct session *ses, char *textin, char *textout, int scroll)
 			continue;
 		}
 
-		if (*pti == ' ' && HAS_BIT(ses->flags, SES_FLAG_WORDWRAP))
+		if (*pti == ' ')
 		{
 			los = pto;
 			lis = pti;
@@ -154,26 +156,16 @@ int word_wrap(struct session *ses, char *textin, char *textout, int scroll)
 			cnt++;
 			ses->cur_col = 1;
 
-			if (!HAS_BIT(ses->flags, SES_FLAG_WORDWRAP))
+			if (pto - los > 15 || !SCROLL(ses))
 			{
-				continue;
-			}
-			else if (pto - los > 15 || !SCROLL(ses))
-			{
-				*pto = '\r';
-				pto++;
-				*pto = '\n';
-				pto++;
+				*pto++ = '\n';
 				los = pto;
 				lis = pti;
 			}
 			else
 			{
 				pto = los;
-				*pto = '\r';
-				pto++;
-				*pto = '\n';
-				pto++;
+				*pto++ = '\n';
 				pti = lis;
 				pti++;
 			}
@@ -189,4 +181,3 @@ int word_wrap(struct session *ses, char *textin, char *textout, int scroll)
 
 	return (cnt + 1);
 }
-
