@@ -27,19 +27,62 @@
 ******************************************************************************/
 
 #include <regex.h>
+#include <sys/types.h>
+#include <pcre.h>
 
 #include "tintin.h"
 
-
 /******************************************************************************
 * Simple regexp that skips vt102 codes and supports the * and ? wildcards     *
 ******************************************************************************/
 
-int regexp(char *exp, char *str, unsigned char cs)
+int match(struct session *ses, char *str, char *exp)
+{
+	char expbuf[BUFFER_SIZE], tmpbuf[BUFFER_SIZE];
+
+	if (HAS_BIT(ses->flags, SES_FLAG_REGEXP))
+	{
+		substitute(ses, exp, tmpbuf, SUB_VAR|SUB_FUN|SUB_ESC);
+
+		sprintf(expbuf, "^%s$", tmpbuf);
+
+		return regexp(str, expbuf);
+	}
+	else
+	{
+		substitute(ses, exp, expbuf, SUB_VAR|SUB_FUN);
+
+		return glob(str, expbuf);
+	}
+}
+
+int find(struct session *ses, char *str, char *exp)
+{
+	char expbuf[BUFFER_SIZE], tmpbuf[BUFFER_SIZE];
+
+	if (HAS_BIT(ses->flags, SES_FLAG_REGEXP))
+	{
+		strip_vt102_codes(str, tmpbuf);
+
+		substitute(ses, exp, expbuf, SUB_VAR|SUB_FUN|SUB_ESC);
+
+		return regexp(tmpbuf, expbuf);
+	}
+	else
+	{
+		substitute(ses, exp, tmpbuf, SUB_VAR|SUB_FUN);
+
+		sprintf(expbuf, "*%s*", tmpbuf);
+
+		return glob(str, expbuf);
+	}
+}
+
+int glob(char *str, char *exp)
 {
 	short cnt;
 
-	for ( ; *exp != 0 ; exp++, str++)
+	while (*exp)
 	{
 		while ((cnt = skip_vt102_codes(str)) != 0)
 		{
@@ -64,12 +107,14 @@ int regexp(char *exp, char *str, unsigned char cs)
 				{
 					return FALSE;
 				}
+				exp++;
+				str++;
 				break;
 
 			case '*':
 				for (cnt = strlen(str) ; cnt >= 0 ; cnt--)
 				{
-					if (regexp(exp + 1, &str[cnt], cs))
+					if (glob(&str[cnt], exp + 1))
 					{
 						return TRUE;
 					}
@@ -77,26 +122,12 @@ int regexp(char *exp, char *str, unsigned char cs)
 				return FALSE;
 
 			case '\\':
-				if (*++exp != *str)
-				{
-					return FALSE;
-				}
-				break;
+				exp++;
 
 			default:
-				if (cs)
+				if (*exp++ != *str++)
 				{
-					if (*exp != *str)
-					{
-						return FALSE;
-					}
-				}
-				else
-				{
-					if (tolower(*exp) != tolower(*str))
-					{
-						return FALSE;
-					}
+					return FALSE;
 				}
 				break;
 		}
@@ -106,84 +137,123 @@ int regexp(char *exp, char *str, unsigned char cs)
 
 DO_COMMAND(do_regexp)
 {
-	char left[BUFFER_SIZE], right[BUFFER_SIZE], true[BUFFER_SIZE], false[BUFFER_SIZE], str[BUFFER_SIZE], exp[BUFFER_SIZE];
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], is_true[BUFFER_SIZE], is_false[BUFFER_SIZE];
 
 	arg = get_arg_in_braces(arg, left,  0);
 	arg = get_arg_in_braces(arg, right, 0);
-	arg = get_arg_in_braces(arg, true,  1);
-	arg = get_arg_in_braces(arg, false, 1);
+	arg = get_arg_in_braces(arg, is_true,  1);
+	arg = get_arg_in_braces(arg, is_false, 1);
 
-	if (*left == 0 || *right == 0 || *true == 0)
+	if (*right == 0 || *is_true == 0)
 	{
-		tintin_printf2(ses, "SYNTAX: #REGEXP {string} {expression} {true} {false}");
+		tintin_printf2(ses, "SYNTAX: #REGEXP {string} {expression} {true} {false}.");
 	}
 	else
 	{
-		substitute(ses, left,  str, SUB_VAR|SUB_FUN);
-		substitute(ses, right, exp, SUB_VAR|SUB_FUN);
+		substitute(ses, left,  left,  SUB_VAR|SUB_FUN);
+		substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
-		if (regex(exp, str))
+		if (regexp_compare(NULL, left, right, 0, SUB_CMD))
 		{
-			ses = pre_parse_input(ses, true, SUB_CMD);
+			substitute(ses, is_true, is_true, SUB_CMD);
+
+			ses = script_driver(ses, -1, is_true);
 		}
-		else if (*false)
+		else if (*is_false)
 		{
-			ses = pre_parse_input(ses, false, SUB_NONE);
+			ses = script_driver(ses, -1, is_false);
 		}
 	}
 	return ses;
 }
-		
-int regex(char *exp, char *str)
+
+int regexp_compare(pcre *nodepcre, char *str, char *exp, int option, int flag)
 {
-	regex_t regex;
-	regmatch_t match[100];
-	int i;
+	pcre *regex;
+	const char *error;
+	int i, j, matches, match[303];
 
-	if (regcomp(&regex, exp, REG_EXTENDED))
+	if (nodepcre == NULL)
+	{
+		regex = pcre_compile(exp, option, &error, &i, NULL);
+	}
+	else
+	{
+		regex = nodepcre;
+	}
+
+	if (regex == NULL)
 	{
 		return FALSE;
 	}
 
-	if (regexec(&regex, str, 100, match, 0))
-	{
-		regfree(&regex);
+	matches = pcre_exec(regex, NULL, str, strlen(str), 0, 0, match, 303);
 
-		return FALSE;
-	}
-
-	for (i = 0 ; i < 100 ; i++)
+	if (matches <= 0)
 	{
-		if (match[i].rm_so != -1)
+		if (nodepcre == NULL)
 		{
-			gtd->cmds[i] = refstring(gtd->cmds[i], "%.*s", match[i].rm_eo - match[i].rm_so, &str[match[i].rm_so]);
+			free(regex);
 		}
-		else
-		{
+		return FALSE;
+	}
+
+	switch (flag)
+	{
+		case SUB_CMD:
+			for (i = 0 ; i < matches ; i++)
+			{
+				gtd->cmds[i] = refstring(gtd->cmds[i], "%.*s", match[i*2+1] - match[i*2], &str[match[i*2]]);
+			}
+			break;
+
+		case SUB_ARG:
+			for (i = 0 ; i < matches ; i++)
+			{
+				j = gtd->args[i];
+
+				gtd->vars[j] = refstring(gtd->vars[j], "%.*s", match[i*2+1] - match[i*2], &str[match[i*2]]);
+			}
 			break;
-		}
 	}
-	regfree(&regex);
+
+	if (nodepcre == NULL)
+	{
+		free(regex);
+	}
 
 	return TRUE;
 }
 
+pcre *regexp_compile(char *exp, int option)
+{
+	const char *error;
+	int i;
+
+	return pcre_compile(exp, option, &error, &i, NULL);
+}
+
+int regexp(char *str, char *exp)
+{
+	return regexp_compare(NULL, str, exp, 0, 0);
+}
+
 /******************************************************************************
 * copy *string into *result, but substitute the various expressions with the  *
 * values they stand for.                                                      *
 ******************************************************************************/
 
-void substitute(struct session *ses, char *string, char *result, int flags)
+int substitute(struct session *ses, char *string, char *result, int flags)
 {
 	struct listnode *node;
-	char temp[BUFFER_SIZE], buf[BUFFER_SIZE], buffer[STRING_SIZE], *pti, *pto, *ptt;
+	char temp[BUFFER_SIZE], buf[BUFFER_SIZE], buffer[BUFFER_SIZE], *pti, *pto, *ptt;
 	char *pte;
 	int i, cnt, flags_neol = flags;
 
 	push_call("substitute(%p,%p,%p,%d)",ses,string,result,flags);
 
 	pti = string;
-	pto = buffer;
+	pto = (string == result) ? buffer : result;
 
 	DEL_BIT(flags_neol, SUB_EOL);
 
@@ -207,19 +277,51 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 			case '\0':
 				if (HAS_BIT(flags, SUB_EOL))
 				{
-					*pto++ = '\r';
+					if (HAS_BIT(ses->flags, SES_FLAG_RUN))
+					{
+						*pto++ = '\r';
+					}
+					else
+					{
+						*pto++ = '\r';
+						*pto++ = '\n';
+					}
+				}
+
+				if (HAS_BIT(flags, SUB_LNF))
+				{
 					*pto++ = '\n';
 				}
+
 				*pto = 0;
 
-				strcpy(result, buffer);
-
 				pop_call();
-				return;
+
+				if (string == result)
+				{
+					strcpy(result, buffer);
+
+					return pto - buffer;
+				}
+				else
+				{
+					return pto - result;
+				}
 
 			case '$':
-				if (HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalnum(pti[1])))
+				if (!VERBATIM && HAS_BIT(flags, SUB_VAR) && (pti[1] == DEFAULT_OPEN || isalnum(pti[1]) || pti[1] == '$'))
 				{
+					if (pti[1] == '$')
+					{
+						while (pti[1] == '$')
+						{
+							*pto++ = *pti++;
+						}
+						pti++;
+
+						continue;
+					}
+
 					if (pti[1] == DEFAULT_OPEN)
 					{
 						for (ptt = buf, i = 2 ; pti[i] && pti[i] != DEFAULT_CLOSE ; i++)
@@ -276,7 +378,8 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 								substitute(ses, temp, temp, SUB_VAR|SUB_FUN);
 
 								get_list_item(ses, node, temp);
-								strcpy(pto, temp);
+
+								substitute(ses, temp, pto, flags_neol - SUB_VAR);
 							}
 						}
 						pto += strlen(pto);
@@ -365,7 +468,39 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 						*pto++ = ';';
 						*pto++ = '5';
 						*pto++ = ';';
-						cnt = 16 + (pti[1] - 'A') + (pti[2] - 'A') * 6 + (pti[3] - 'A') * 36;
+						cnt = 16 + (pti[1] - 'A') * 36 + (pti[2] - 'A') * 6 + (pti[3] - 'A');
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = 'm';
+						pti += 5;
+					}
+					else if (pti[1] == 'g' && isdigit(pti[2]) && isdigit(pti[3]) && pti[4] == '>')
+					{
+						*pto++ = ESCAPE;
+						*pto++ = '[';
+						*pto++ = '3';
+						*pto++ = '8';
+						*pto++ = ';';
+						*pto++ = '5';
+						*pto++ = ';';
+						cnt = 232 + (pti[2] - '0') * 10 + (pti[3] - '0');
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = 'm';
+						pti += 5;
+					}
+					else if (pti[1] == 'G' && isdigit(pti[2]) && isdigit(pti[3]) && pti[4] == '>')
+					{
+						*pto++ = ESCAPE;
+						*pto++ = '[';
+						*pto++ = '4';
+						*pto++ = '8';
+						*pto++ = ';';
+						*pto++ = '5';
+						*pto++ = ';';
+						cnt = 232 + (pti[2] - '0') * 10 + (pti[3] - '0');
 						*pto++ = '0' + cnt / 100;
 						*pto++ = '0' + cnt % 100 / 10;
 						*pto++ = '0' + cnt % 10;
@@ -384,7 +519,7 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 				break;
 
 			case '@':
-				if (HAS_BIT(flags, SUB_FUN))
+				if (!VERBATIM && HAS_BIT(flags, SUB_FUN))
 				{
 					for (ptt = temp, i = 1 ; isalnum(pti[i]) || pti[i] == '_' ; i++)
 					{
@@ -403,9 +538,9 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 
 					pti = get_arg_in_braces(&pti[i], temp, FALSE);
 
-					substitute(ses, temp, buf, flags_neol);
+					substitute(ses, temp, buf, SUB_VAR|SUB_FUN);
 
-					show_debug(ses, LIST_FUNCTION, "#FUNCTION DEBUG: {%s} {%s}", node->left, buf);
+					show_debug(ses, LIST_FUNCTION, "#DEBUG FUNCTION {%s}", node->left);
 
 					RESTRING(gtd->vars[0], buf);
 
@@ -423,13 +558,11 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 						}
 					}
 
-					pre_parse_input(ses, node->right, SUB_ARG);
+					substitute(ses, node->right, buf, SUB_ARG);
 
-					DEL_BIT(ses->flags, SES_FLAG_BREAK);
+					script_driver(ses, LIST_FUNCTION, buf);
 
-					substitute(ses, "$result", pto, SUB_VAR);
-
-					show_debug(ses, LIST_FUNCTION, "#FUNCTION DEBUG: {%s} {%s}", node->left, pto);
+					substitute(ses, "$result", pto, flags_neol|SUB_VAR);
 
 					pto += strlen(pto);
 				}
@@ -448,14 +581,7 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 						{
 							*pto++ = *pti++;
 						}
-						if (isdigit(pti[1]))
-						{
-							pti++;
-						}
-						else
-						{
-							*pto++ = *pti++;
-						}
+						pti++;
 					}
 					else
 					{
@@ -490,6 +616,20 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 										*pto++ = '\\';
 										break;
 
+									case '{':
+										*pto++ = '\\';
+										*pto++ = 'x';
+										*pto++ = '7';
+										*pto++ = 'B';
+										break;
+
+									case '}':
+										*pto++ = '\\';
+										*pto++ = 'x';
+										*pto++ = '7';
+										*pto++ = 'D';
+										break;
+
 									default:
 										*pto++ = *ptt;
 										break;
@@ -510,29 +650,6 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 				}
 				break;
 
-			case '^':
-				if (HAS_BIT(flags, SUB_ANC) && (pti == string || pti[1] == 0))
-				{
-					pti++;
-				}
-				else
-				{
-					*pto++ = *pti++;
-				}
-				break;
-
-			case '~':
-				if (HAS_BIT(flags, SUB_ANC) && pti == string)
-				{
-					string++;
-					pti++;
-				}
-				else
-				{
-					*pto++ = *pti++;
-				}
-				break;
-
 			case '&':
 				if (HAS_BIT(flags, SUB_CMD) && (isdigit(pti[1]) || pti[1] == '&'))
 				{
@@ -569,7 +686,7 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 				break;
 
 			case '\\':
-				if (HAS_BIT(flags, SUB_ESC))
+				if (!VERBATIM && HAS_BIT(flags, SUB_ESC))
 				{
 					pti++;
 
@@ -581,6 +698,13 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 						case 'b':
 							*pto++ = '\b';
 							break;
+						case 'c':
+							if (pti[1])
+							{
+								pti++;
+								*pto++ = *pti % 32;
+							}
+							break;
 						case 'e':
 							*pto++ = '\033';
 							break;
@@ -594,22 +718,24 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 							*pto++ = '\t';
 							break;
 						case 'x':
-							pti++;
-
-							if (pti[0] && pti[1])
+							if (pti[1] && pti[2])
 							{
+								pti++;
 								*pto++ = hex_number(pti);
 								pti++;
 							}
 							break;
-						case '[':
-							*pto++ = '{';
-							break;
-						case ']':
-							*pto++ = '}';
+						case '0':
+							if (pti[1] && pti[2])
+							{
+								pti++;
+								*pto++ = oct_number(pti);
+								pti++;
+							}
 							break;
 						case '\0':
 							DEL_BIT(flags, SUB_EOL);
+							DEL_BIT(flags, SUB_LNF);
 							continue;
 						default:
 							*pto++ = *pti;
@@ -632,179 +758,369 @@ void substitute(struct session *ses, char *string, char *result, int flags)
 
 
 /******************************************************************************
-* Calls action_regexp checking if the string matches, and automatically fills *
-* in the text represented by the wildcards on success. Per default the entire *
-* string is compared unless it starts or ends with the ^ character.           *
+* Calls tintin_regexp checking if the string matches, and automatically fills *
+* in the text represented by the wildcards on success.                        *
 ******************************************************************************/
 
-int check_one_action(char *line, char *original, char *action, struct session *ses)
+int check_one_regexp(struct session *ses, struct listnode *node, char *line, char *original, int option)
 {
-	char result[BUFFER_SIZE];
-	char *ptr, *ptl;
+	char *exp, *str;
 
-	substitute(ses, action, result, SUB_VAR|SUB_FUN|SUB_ESC);
-
-	ptr = result;
-
-	if (*ptr == '~')
+	if (node->regex == NULL)
 	{
-		ptr++;
-		ptl = original;
+		char result[BUFFER_SIZE];
+
+		substitute(ses, node->left, result, SUB_VAR|SUB_FUN);
+
+		exp = result;
 	}
 	else
 	{
-		ptl = line;
+		exp = node->left;
 	}
 
-	if (*ptr == '^')
+	if (HAS_BIT(node->flags, NODE_FLAG_META))
 	{
-		ptr++;
-		return action_regexp(ptr, ptl, 255);
+		exp++;
+		str = original;
 	}
+	else
+	{
+		str = line;
+	}	
 
-	if (ptr[0] == '%' && isdigit(ptr[1]))
-	{
-		return action_regexp(ptr, ptl, 255);
-	}
-
-	while (*ptl)
-	{
-		if ((*ptr == '[' || *ptr == *ptl) && action_regexp(ptr, ptl, 255))
-		{
-			return TRUE;
-		}
-		ptl++;
-	}
-	return FALSE;
+	return tintin_regexp(node->regex, str, exp, option);
 }
 
+/*
+	Keep synched with tintin_regexp_compile
+*/
 
-int action_regexp(char *exp, char *str, unsigned char arg)
+int tintin_regexp(pcre *nodepcre, char *str, char *exp, int option)
 {
-	short cnt, cnt2;
+	char out[BUFFER_SIZE], *pti, *pto;
+	int arg, var;
 
-	thestart:
+	arg = var = 1;
 
-	while (*exp)
+	pti = exp;
+	pto = out;
+
+	while (*pti)
 	{
 
 #ifdef BIG5
-		if (*exp & 0x80 && *str & 0x80)
+		if (*pti & 0x80)
 		{
-			exp++;
-			str++;
-			if (*exp++ != *str++)
-			{
-				return FALSE;
-			}
+			*pto++ = *pti++;
+			*pto++ = *pti++;
 			continue;
 		}
 #endif
-
-		switch (exp[0])
+		switch (pti[0])
 		{
-			case '%':
-				if (isdigit(exp[1]))
-				{
-					arg = isdigit(exp[2]) ? (exp[1] - '0') * 10 + exp[2] - '0' : exp[1] - '0';
-
-					if (exp[isdigit(exp[2]) ? 3 : 2] == 0)
-					{
-						RESTRING(gtd->vars[arg], str);
-
-						return TRUE;
-					}
-
-					for (cnt = 0 ; str[cnt] ; cnt++)
-					{
-						if (action_regexp(exp + (isdigit(exp[2]) ? 3 : 2), &str[cnt], arg))
-						{
-							gtd->vars[arg] = refstring(gtd->vars[arg], "%.*s", cnt, str);
-
-							return TRUE;
-						}
-					}
-					return FALSE;
-				}
+			case '\\':
+				*pto++ = *pti++;
+				*pto++ = *pti++;
 				break;
 
-			case '^':
-				if (exp[1] == 0)
-				{
-					return (exp[1] == str[0]);
-				}
+			case '{':
+				gtd->args[up(var)] = up(arg);
+				*pto++ = '(';
+				pti = get_arg_in_braces(pti, pto, TRUE);
+				pto += strlen(pto);
+				*pto++ = ')';
 				break;
 
 			case '[':
-				if (*str != '[' && regexp("[*|*]*", exp, TRUE))
+			case ']':
+			case '(':
+			case ')':
+			case '|':
+			case '.':
+			case '?':
+			case '+':
+			case '*':
+				*pto++ = '\\';
+				*pto++ = *pti++;
+				break;
+
+			case '%':
+				switch (pti[1])
 				{
-					cnt  = 1;
-					cnt2 = 0;
+					case '0':
+					case '1':
+					case '2':
+					case '3':
+					case '4':
+					case '5':
+					case '6':
+					case '7':
+					case '8':
+					case '9':
+						arg = isdigit(pti[2]) ? (pti[1] - '0') * 10 + (pti[2] - '0') : pti[1] - '0';
+						gtd->args[up(var)] = up(arg);
+						pti += isdigit(pti[2]) ? 3 : 2;
+						strcpy(pto, *pti == 0 ? "(.*)" : "(.*?)");
+						pto += strlen(pto);
+						break;
 
-					while (TRUE)
-					{
-						if (exp[cnt] == '|' || exp[cnt] == ']')
-						{
-							arg = (arg == 99) ? arg : arg + 1;
-							cnt--;
-							cnt2 = 0;
+					case 'w':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([a-zA-Z]*)" : "([a-zA-Z]*?)");
+						pto += strlen(pto);
+						break;
 
-							while (exp[cnt] != '|' && exp[cnt] != '[')
-							{
-								cnt--;
-								cnt2++;
-								str++;
-							}
-							gtd->vars[arg] = refstring(gtd->vars[arg], "%.*s", cnt2, &exp[cnt+1]);
+					case 'W':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([^a-zA-Z]*)" : "([^a-zA-Z]*?)");
+						pto += strlen(pto);
+						break;
 
-							while (*exp != ']')
-							{
-								exp++;
-							}
-							exp++;
+					case 'd':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([0-9]*)" : "([0-9]*?)");
+						pto += strlen(pto);
+						break;
 
-							goto thestart;
-						}
+					case 'D':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([^0-9]*)" : "([^0-9]*?)");
+						pto += strlen(pto);
+						break;
 
-						if (str[cnt2] == 0)
-						{
-							break;
-						}
+					case 's':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(\\s*)" : "(\\s*?)");
+						pto += strlen(pto);
+						break;
 
-						if (exp[cnt] == str[cnt2])
-						{
-							cnt++;
-							cnt2++;
-							continue;
-						}
-						else
-						{
-							cnt2 = 0;
+					case 'S':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(\\S*)" : "(\\S*?)");
+						pto += strlen(pto);
+						break;
 
-							while (exp[cnt])
-							{
-								if (exp[cnt] == ']')
-								{
-									return FALSE;
-								}
-								if (exp[cnt++] == '|')
-								{
-									break;
-								}
+					case '?':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(.?)" : "(.??)");
+						pto += strlen(pto);
+						break;
 
-							}
-						}
-					}
+					case '*':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(.*)" : "(.*?)");
+						pto += strlen(pto);
+						break;
+
+					case '+':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(.+)" : "(.+?)");
+						pto += strlen(pto);
+						break;
+
+					case '.':
+						gtd->args[up(var)] = up(arg);
+						pti += 2;
+						strcpy(pto, "(.)");
+						pto += strlen(pto);
+						break;
+
+					case '%':
+						*pto++ = *pti++;
+						pti++;
+						break;
+
+					default:
+						*pto++ = *pti++;
+						break;
 				}
 				break;
-		}
 
-		if (*exp != *str)
-		{
-			return FALSE;
+			default:
+				*pto++ = *pti++;
+				break;
 		}
-		exp++;
-		str++;
 	}
-	return TRUE;
+	*pto = 0;
+
+	return regexp_compare(nodepcre, str, out, option, SUB_ARG);
+}
+
+pcre *tintin_regexp_compile(struct listnode *node, char *exp, int option)
+{
+	char out[BUFFER_SIZE], *pti, *pto;
+
+	pti = exp;
+	pto = out;
+
+	if (*pti == '~')
+	{
+		pti++;
+		SET_BIT(node->flags, NODE_FLAG_META);
+	}
+
+	while (*pti)
+	{
+
+#ifdef BIG5
+		if (*pti & 0x80)
+		{
+			*pto++ = *pti++;
+			*pto++ = *pti++;
+			continue;
+		}
+#endif
+		switch (pti[0])
+		{
+			case '\\':
+				*pto++ = *pti++;
+				*pto++ = *pti++;
+				break;
+
+			case '{':
+				*pto++ = '(';
+				pti = get_arg_in_braces(pti, pto, TRUE);
+				while (*pto)
+				{
+					if (pto[0] == '$' || pto[0] == '@')
+					{
+						if (pto[1])
+						{
+							return NULL;
+						}
+					}
+					pto++;
+				}
+				*pto++ = ')';
+				break;
+
+			case '$':
+			case '@':
+				if (pti[1])
+				{
+					return NULL;
+				}
+				*pto++ = *pti++;
+				break;
+
+			case '[':
+			case ']':
+			case '(':
+			case ')':
+			case '|':
+			case '.':
+			case '?':
+			case '+':
+			case '*':
+				*pto++ = '\\';
+				*pto++ = *pti++;
+				break;
+
+			case '%':
+				switch (pti[1])
+				{
+					case '0':
+					case '1':
+					case '2':
+					case '3':
+					case '4':
+					case '5':
+					case '6':
+					case '7':
+					case '8':
+					case '9':
+						pti += isdigit(pti[2]) ? 3 : 2;
+						strcpy(pto, *pti == 0 ? "(.*)" : "(.*?)");
+						pto += strlen(pto);
+						break;
+
+					case 'w':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([a-zA-Z]*)" : "([a-zA-Z]*?)");
+						pto += strlen(pto);
+						break;
+
+					case 'W':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([^a-zA-Z]*)" : "([^a-zA-Z]*?)");
+						pto += strlen(pto);
+						break;
+
+					case 'd':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([0-9]*)" : "([0-9]*?)");
+						pto += strlen(pto);
+						break;
+
+					case 'D':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "([^0-9]*)" : "([^0-9]*?)");
+						pto += strlen(pto);
+						break;
+
+					case 's':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(\\s*)" : "(\\s*?)");
+						pto += strlen(pto);
+						break;
+
+					case 'S':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(\\S*)" : "(\\S*?)");
+						pto += strlen(pto);
+						break;
+
+					case '?':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(.?)" : "(.??)");
+						pto += strlen(pto);
+						break;
+
+					case '*':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(.*)" : "(.*?)");
+						pto += strlen(pto);
+						break;
+
+					case '+':
+						pti += 2;
+						strcpy(pto, *pti == 0 ? "(.+)" : "(.+?)");
+						pto += strlen(pto);
+						break;
+
+					case '.':
+						pti += 2;
+						strcpy(pto, "(.)");
+						pto += strlen(pto);
+						break;
+
+					case '%':
+						*pto++ = *pti++;
+						pti++;
+						break;
+
+					default:
+						*pto++ = *pti++;
+						break;
+				}
+				break;
+
+			default:
+				*pto++ = *pti++;
+				break;
+		}
+	}
+	*pto = 0;
+
+	return regexp_compile(out, option);
 }
