@@ -42,18 +42,29 @@ int is_abbrev(char *s1, char *s2)
 	{
 		return (FALSE);
 	}
-	return(!strncasecmp(s2, s1, strlen(s1)));
+	return (!strncasecmp(s2, s1, strlen(s1)));
 }
 
-int is_suffix(char *s1, char *s2)
+
+int is_color_code(char *str)
 {
-	int sl1, sl2;
-
-	sl1 = strlen(s1);
-	sl2 = strlen(s2);
-
-	return (sl1 <= sl2 && !strcasecmp(s1, s2 + sl2 - sl1)) ? FALSE : TRUE;
+	while (*str)
+	{
+		if (*str++ == '<')
+		{
+			if (isalnum(*str++) && isalnum(*str++) && isalnum(*str++))
+			{
+				if (*str++ == '>')
+				{
+					continue;
+				}
+			}
+		}
+		return FALSE;
+	}
+	return TRUE;
 }
+
 /*
 	return TRUE if the string is a number
 */
@@ -62,14 +73,21 @@ int is_number(char *str)
 {
 	int i;
 
-	if (!isdigit(str[0]) && str[0] != '-' && str[0] != '+')
+	i = 0;
+
+	if (str[i] == '-' || str[i] == '+')
+	{
+		i++;
+	}
+
+	if (!isdigit(str[i]))
 	{
 		return FALSE;
 	}
 
 	for (i = 1 ; str[i] != 0 ; i++)
 	{
-		if (!isdigit(str[i]))
+		if (!isdigit(str[i]) && str[i] != '.')
 		{
 			return FALSE;
 		}
@@ -79,17 +97,17 @@ int is_number(char *str)
 
 int hex_number(char *str)
 {
-	int value = '?';
+	int value = 0;
 
 	if (str)
 	{
 		if (isdigit(*str))
 		{
-			value = (*str - '0') << 4;
+			value += 16 * (*str - '0');
 		}
 		else
 		{
-			value = (*str - 'A' + 10) << 4;
+			value += 16 * (toupper(*str) - 'A' + 10);
 		}
 		str++;
 	}
@@ -102,7 +120,32 @@ int hex_number(char *str)
 		}
 		else
 		{
-			value += *str - 'A' + 10;
+			value += toupper(*str) - 'A' + 10;
+		}
+		str++;
+	}
+
+	return value;
+}
+
+int oct_number(char *str)
+{
+	int value = 0;
+
+	if (str)
+	{
+		if (isdigit(*str))
+		{
+			value += 8 * (*str - '0');
+		}
+		str++;
+	}
+
+	if (str)
+	{
+		if (isdigit(*str))
+		{
+			value += *str - '0';
 		}
 		str++;
 	}
@@ -127,7 +170,6 @@ long long utime()
 	return gtd->time;
 }
 
-
 char *capitalize(char *str)
 {
 	static char outbuf[BUFFER_SIZE];
@@ -142,6 +184,29 @@ char *capitalize(char *str)
 	return outbuf;
 }
 
+char *ntos(long long number)
+{
+	static char outbuf[100][NUMBER_SIZE];
+	static int cnt;
+
+	cnt = (cnt + 1) % 100;
+
+	sprintf(outbuf[cnt], "%lld", number);
+
+	return outbuf[cnt];
+}
+
+char *indent(int cnt)
+{
+	static char outbuf[BUFFER_SIZE];
+
+	memset(outbuf, '\t', cnt);
+
+	outbuf[cnt] = 0;
+
+	return outbuf;
+}
+
 void cat_sprintf(char *dest, char *fmt, ...)
 {
 	char buf[STRING_SIZE];
@@ -170,6 +235,23 @@ void ins_sprintf(char *dest, char *fmt, ...)
 	strcat(dest, tmp);
 }
 
+int str_suffix(char *str1, char *str2)
+{
+	int len1, len2;
+
+	len1 = strlen(str1);
+	len2 = strlen(str2);
+
+	if (len1 >= len2)
+	{
+		if (!strcasecmp(str1 + len1 - len2, str2))
+		{
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
 void show_message(struct session *ses, int index, char *format, ...)
 {
 	struct listroot *root;
@@ -219,7 +301,7 @@ void show_message(struct session *ses, int index, char *format, ...)
 	{
 		if (ses->logfile)
 		{
-			logit(ses, buf, ses->logfile);
+			logit(ses, buf, ses->logfile, TRUE);
 		}
 	}
 	pop_call();
@@ -232,15 +314,16 @@ void show_debug(struct session *ses, int index, char *format, ...)
 	char buf[STRING_SIZE];
 	va_list args;
 
+	push_call("show_debug(%p,%p,%p)",ses,index,format);
+
 	root = ses->list[index];
 
 	if (!HAS_BIT(root->flags, LIST_FLAG_DEBUG) && !HAS_BIT(root->flags, LIST_FLAG_LOG))
 	{
+		pop_call();
 		return;
 	}
 
-	push_call("show_debug(%p,%p,%p)",ses,index,format);		
-
 	va_start(args, format);
 
 	vsprintf(buf, format, args);
@@ -259,7 +342,7 @@ void show_debug(struct session *ses, int index, char *format, ...)
 	{
 		if (ses->logfile)
 		{
-			logit(ses, buf, ses->logfile);
+			logit(ses, buf, ses->logfile, TRUE);
 		}
 	}
 	pop_call();
@@ -275,7 +358,7 @@ void tintin_header(struct session *ses, char *format, ...)
 	vsprintf(arg, format, args);
 	va_end(args);
 
-	if (strlen(arg) > gtd->ses->cols - 2)
+	if ((int) strlen(arg) > gtd->ses->cols - 2)
 	{
 		arg[gtd->ses->cols - 2] = 0;
 	}
@@ -329,6 +412,54 @@ void tintin_printf(struct session *ses, char *format, ...)
 	tintin_puts(ses, buf);
 }
 
+/*
+	Like tintin_puts2, but no color codes added
+*/
+
+void tintin_puts3(struct session *ses, char *string)
+{
+	char output[STRING_SIZE];
+
+	if (ses == NULL)
+	{
+		ses = gtd->ses;
+	}
+
+	if (!HAS_BIT(gtd->ses->flags, SES_FLAG_VERBOSE) && gtd->quiet)
+	{
+		return;
+	}
+
+	if (strip_vt102_strlen(ses->more_output) != 0)
+	{
+		sprintf(output, "\n%s", string);
+	}
+	else
+	{
+		sprintf(output, "%s", string);
+	}
+
+	add_line_buffer(ses, output, FALSE);
+
+	if (ses != gtd->ses)
+	{
+		return;
+	}
+
+	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
+	{
+		save_pos(ses);
+		goto_rowcol(ses, ses->bot_row, 1);
+	}
+
+	printline(ses, output, FALSE);
+
+	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
+	{
+		restore_pos(ses);
+	}
+}
+
 /*
 	output to screen should go through this function
 	the output is NOT checked for actions or anything
@@ -350,7 +481,7 @@ void tintin_puts2(struct session *ses, char *string)
 
 	if (strip_vt102_strlen(ses->more_output) != 0)
 	{
-		sprintf(output, "\r\n\033[0m%s\033[0m", string);
+		sprintf(output, "\n\033[0m%s\033[0m", string);
 	}
 	else
 	{
@@ -409,8 +540,6 @@ void tintin_puts(struct session *ses, char *string)
 
 void syserr(char *msg)
 {
-	extern int errno;
-
 	char s[128], *syserrmsg;
 
 	syserrmsg = strerror(errno);
