@@ -37,13 +37,16 @@ DO_COMMAND(do_variable)
 	root = ses->list[LIST_VARIABLE];
 
 	arg = get_arg_in_braces(arg, left, FALSE);
-	arg = get_arg_in_braces(arg, right, TRUE);
+	substitute(ses, left, left, SUB_VAR|SUB_FUN);
+
+	get_arg_in_braces(arg, right, TRUE);
+	substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
 	if (*left == 0)
 	{
 		show_list(ses, root, LIST_VARIABLE);
 	}
-	else if (*left && *right == 0)
+	else if (*left && *arg == 0)
 	{
 		if (show_node_with_wild(ses, left, LIST_VARIABLE) == FALSE)
 		{
@@ -53,6 +56,8 @@ DO_COMMAND(do_variable)
 	else
 	{
 		internal_variable(ses, "{%s} {%s}", left, right);
+
+		show_message(ses, LIST_VARIABLE, "#OK. VARIABLE {%s} HAS BEEN SET TO {%s}.", left, right);
 	}
 	return ses;
 }
@@ -123,51 +128,53 @@ void internal_variable(struct session *ses, char *format, ...)
 		updatenode_list(ses, left, right, "", LIST_VARIABLE);
 	}
 
-	show_message(ses, LIST_VARIABLE, "#OK. VARIABLE {%s} HAS BEEN SET TO {%s}.", left, right);
+	show_debug(ses, LIST_VARIABLE, "#OK. VARIABLE {%s} HAS BEEN SET TO {%s}.", left, right);
 }
 
 
 
-DO_COMMAND(do_replacestring)
+DO_COMMAND(do_replace)
 {
-	char var[BUFFER_SIZE], old[BUFFER_SIZE], new[BUFFER_SIZE], buf[STRING_SIZE], *pti, *ptr, *pto;
+	char var[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE], buf[STRING_SIZE], *pti, *ptr, *pto;
 	struct listroot *root;
 	struct listnode *node;
 
 	root = ses->list[LIST_VARIABLE];
 
 	arg = get_arg_in_braces(arg, var, FALSE);
-	arg = get_arg_in_braces(arg, old, FALSE);
-	arg = get_arg_in_braces(arg, new, TRUE);
+	arg = get_arg_in_braces(arg, left, FALSE);
+	substitute(ses, left, left, SUB_VAR|SUB_FUN|SUB_ESC);
+	arg = get_arg_in_braces(arg, right, TRUE);
+	substitute(ses, right, right, SUB_VAR|SUB_FUN);
 
-	if (*var == 0 || *old == 0)
+	if (*var == 0 || *left == 0)
 	{
-		show_message(ses, LIST_VARIABLE, "#Syntax: #replacestring <var> <oldtext> <newtext>", ses);
+		show_message(ses, LIST_VARIABLE, "#Syntax: #replace <var> <old text> <new text>", ses);
 	}
 	else	if ((node = searchnode_list(root, var)) == NULL)
 	{
-		show_message(ses, LIST_VARIABLE, "#REPLACESTRING: VARIABLE {%s} NOT FOUND.", var);
+		show_message(ses, LIST_VARIABLE, "#REPLACE: VARIABLE {%s} NOT FOUND.", var);
 	}
-	else if ((ptr = strstr(node->right, old)) == NULL)
+	else if ((ptr = strstr(node->right, left)) == NULL)
 	{
-		show_message(ses, LIST_VARIABLE, "#REPLACESTRING: {%s} NOT FOUND IN {%s}", old, node->right);
+		show_message(ses, LIST_VARIABLE, "#REPLACE: {%s} NOT FOUND IN {%s}.", left, node->right);
 	}
 	else
 	{
 		pti = node->right;
 		pto = buf;
 
-		while ((ptr = strstr(pti, old)) != NULL)
+		while ((ptr = strstr(pti, left)) != NULL)
 		{
 			while (pti != ptr)
 			{
 				*pto++ = *pti++;
 			}
 
-			strcpy(pto, new);
-			pto += strlen(new);
+			strcpy(pto, right);
+			pto += strlen(right);
 
-			pti += strlen(old);
+			pti += strlen(left);
 		}
 
 		while (*pti)
@@ -214,7 +221,7 @@ void headerstring(char *str)
 {
 	char buf[BUFFER_SIZE];
 
-	if (strlen(str) > gtd->ses->cols - 2)
+	if ((int) strlen(str) > gtd->ses->cols - 2)
 	{
 		str[gtd->ses->cols - 2] = 0;
 	}
@@ -387,20 +394,48 @@ int stringlength(struct session *ses, char *str)
 	return strip_vt102_strlen(str);
 }
 
+void timestring(struct session *ses, char *str)
+{
+	char left[BUFFER_SIZE], right[BUFFER_SIZE], *arg;
+
+	struct tm timeval_tm;
+	time_t    timeval_t;
+
+	arg = get_arg_in_braces(str, left, TRUE);
+	arg = get_arg_in_braces(arg, right, TRUE);
+
+	if (*right)
+	{
+		timeval_t = (time_t) atoll(right);
+	}
+	else
+	{
+		timeval_t = (time_t) time(NULL);
+	}
+
+	timeval_tm = *localtime(&timeval_t);
+
+	strftime(str, BUFFER_SIZE, left, &timeval_tm);
+}
+
 DO_COMMAND(do_format)
 {
-	char destvar[BUFFER_SIZE], format[BUFFER_SIZE], argument[BUFFER_SIZE], arglist[20][BUFFER_SIZE], *ptf;
+	char temp[BUFFER_SIZE], destvar[BUFFER_SIZE], format[BUFFER_SIZE], argument[BUFFER_SIZE], arglist[20][BUFFER_SIZE], *ptf, *ptt;
 	struct tm timeval_tm;
 	time_t    timeval_t;
 	int i;
 
+	substitute(ses, arg, temp, SUB_VAR|SUB_FUN);
+
+	arg = temp;
+
 	arg = get_arg_in_braces(arg, destvar,  FALSE);
 	arg = get_arg_in_braces(arg, format,   FALSE);
 	arg = get_arg_in_braces(arg, argument, TRUE);
 
 	if (*destvar == 0 || *format == 0)
 	{
-		tintin_printf2(ses, "#SYNTAX: #format {variable} {format} {arguments}");
+		tintin_printf2(ses, "#SYNTAX: #format {variable} {format} {arguments}.");
 
 		return ses;
 	}
@@ -423,7 +458,9 @@ DO_COMMAND(do_format)
 
 		if (*ptf == '%')
 		{
-			ptf++;
+			ptt = temp;
+
+			*ptt++ = *ptf++;
 
 			if (*ptf == 0)
 			{
@@ -437,7 +474,7 @@ DO_COMMAND(do_format)
 			{
 				while (!isalpha(*ptf))
 				{
-					ptf++;
+					*ptt++ = *ptf++;
 				}
 
 				if (*ptf == 0)
@@ -448,7 +485,7 @@ DO_COMMAND(do_format)
 				switch (*ptf)
 				{
 					case 'a':
-						sprintf(arglist[i], "%c", atoi(arglist[i]));
+						sprintf(arglist[i], "%c", (char) get_number(ses, arglist[i]));
 						break;
 
 					case 'c':
@@ -456,9 +493,8 @@ DO_COMMAND(do_format)
 						break;
 
 					case 'd':
-						timeval_t  = (time_t) atoi(arglist[i]);
-						timeval_tm = *localtime(&timeval_t);
-						strftime(arglist[i], BUFFER_SIZE, "%d-%m-%Y", &timeval_tm);
+						strcpy(ptt, "lld");
+						sprintf(arglist[i], temp, (long long) get_number(ses, arglist[i]));
 						break;
 
 					case 'h':
@@ -489,9 +525,7 @@ DO_COMMAND(do_format)
 						break;
 
 					case 't':
-						timeval_t  = (time_t) atoi(arglist[i]);
-						timeval_tm = *localtime(&timeval_t);
-						strftime(arglist[i], BUFFER_SIZE, "%H:%M", &timeval_tm);
+						timestring(ses, arglist[i]);
 						break;
 
 					case 'u':
@@ -508,7 +542,7 @@ DO_COMMAND(do_format)
 						break;
 
 					case 'D':
-						timeval_t  = (time_t) atoi(arglist[i]);
+						timeval_t  = (time_t) *arglist[i] ? atoi(arglist[i]) : time(NULL);
 						timeval_tm = *localtime(&timeval_t);
 						strftime(arglist[i], BUFFER_SIZE, "%d", &timeval_tm);
 						break;
@@ -522,7 +556,7 @@ DO_COMMAND(do_format)
 						break;
 
 					case 'M':
-						timeval_t  = (time_t) atoi(arglist[i]);
+						timeval_t  = (time_t) *arglist[i] ? atoi(arglist[i]) : time(NULL);
 						timeval_tm = *localtime(&timeval_t);
 						strftime(arglist[i], BUFFER_SIZE, "%m", &timeval_tm);
 						break;
@@ -540,13 +574,13 @@ DO_COMMAND(do_format)
 						break;
 
 					case 'Y':
-						timeval_t  = (time_t) atoi(arglist[i]);
+						timeval_t  = (time_t) *arglist[i] ? atoi(arglist[i]) : time(NULL);
 						timeval_tm = *localtime(&timeval_t);
 						strftime(arglist[i], BUFFER_SIZE, "%Y", &timeval_tm);
 						break;
 
 					default:
-						show_message(ses, LIST_VARIABLE, "#FORMAT: UNKNOWN ARGUMENT {%%%c}", *ptf);
+						show_message(ses, LIST_VARIABLE, "#FORMAT: UNKNOWN ARGUMENT {%%%c}.", *ptf);
 						break;
 				}
 				*ptf = 's';
@@ -557,7 +591,7 @@ DO_COMMAND(do_format)
 
 	sprintf(argument, format, arglist[0], arglist[1], arglist[2], arglist[3], arglist[4], arglist[5], arglist[6], arglist[7], arglist[8], arglist[9], arglist[10], arglist[11], arglist[12], arglist[13], arglist[14], arglist[15], arglist[16], arglist[17], arglist[18], arglist[19]);
 
-/*	show_message(ses, LIST_VARIABLE, "#FORMAT: $%s IS NOW SET TO {%s}", destvar, argument); */
+/*	show_message(ses, LIST_VARIABLE, "#FORMAT: $%s IS NOW SET TO {%s}.", destvar, argument); */
 
 	internal_variable(ses, "{%s} {%s}", destvar, argument);
 
